# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    AlreadyExistsError,
    AlreadyExistsErrorTag,
    BadRequestErrorBody,
    BadRequestErrorBody_HttpApiDecodeError,
    BadRequestErrorBody_ImmutableResourceError,
    ClickHouseError,
    DatabaseError,
    DatabaseErrorTag,
    Date,
    HttpApiDecodeError,
    HttpApiDecodeErrorTag,
    ImmutableResourceError,
    InternalServerErrorBody,
    InternalServerErrorBody_ClickHouseError,
    InternalServerErrorBody_DatabaseError,
    Issue,
    IssueTag,
    NotFoundErrorBody,
    NotFoundErrorTag,
    NumberFromString,
    PermissionDeniedError,
    PermissionDeniedErrorTag,
    PlanLimitExceededError,
    PlanLimitExceededErrorTag,
    PricingUnavailableError,
    PropertyKey,
    PropertyKeyKey,
    PropertyKeyKeyTag,
    RateLimitError,
    RateLimitErrorTag,
    ServiceUnavailableErrorBody,
    ServiceUnavailableErrorTag,
    StripeError,
    SubscriptionPastDueError,
    SubscriptionPastDueErrorTag,
    UnauthorizedErrorBody,
    UnauthorizedErrorTag,
)
from .errors import (
    BadRequestError,
    ConflictError,
    ForbiddenError,
    InternalServerError,
    NotFoundError,
    PaymentRequiredError,
    ServiceUnavailableError,
    TooManyRequestsError,
    UnauthorizedError,
)
from . import (
    annotations,
    api_keys,
    docs,
    environments,
    functions,
    health,
    organization_invitations,
    organization_memberships,
    organizations,
    project_memberships,
    projects,
    tags,
    token_cost,
    traces,
)
from .annotations import (
    AnnotationsCreateRequestLabel,
    AnnotationsCreateResponse,
    AnnotationsCreateResponseLabel,
    AnnotationsGetResponse,
    AnnotationsGetResponseLabel,
    AnnotationsListRequestLabel,
    AnnotationsListResponse,
    AnnotationsListResponseAnnotationsItem,
    AnnotationsListResponseAnnotationsItemLabel,
    AnnotationsUpdateRequestLabel,
    AnnotationsUpdateResponse,
    AnnotationsUpdateResponseLabel,
)
from .api_keys import (
    ApiKeysCreateResponse,
    ApiKeysGetResponse,
    ApiKeysListAllForOrgResponseItem,
    ApiKeysListResponseItem,
)
from .client import AsyncMirascope, Mirascope
from .environment import MirascopeEnvironment
from .environments import (
    EnvironmentsCreateResponse,
    EnvironmentsGetAnalyticsResponse,
    EnvironmentsGetAnalyticsResponseTopFunctionsItem,
    EnvironmentsGetAnalyticsResponseTopModelsItem,
    EnvironmentsGetResponse,
    EnvironmentsListResponseItem,
    EnvironmentsUpdateResponse,
)
from .functions import (
    FunctionsCreateRequestDependenciesValue,
    FunctionsCreateResponse,
    FunctionsCreateResponseDependenciesValue,
    FunctionsFindByHashResponse,
    FunctionsFindByHashResponseDependenciesValue,
    FunctionsGetByEnvResponse,
    FunctionsGetByEnvResponseDependenciesValue,
    FunctionsGetResponse,
    FunctionsGetResponseDependenciesValue,
    FunctionsListByEnvResponse,
    FunctionsListByEnvResponseFunctionsItem,
    FunctionsListByEnvResponseFunctionsItemDependenciesValue,
    FunctionsListResponse,
    FunctionsListResponseFunctionsItem,
    FunctionsListResponseFunctionsItemDependenciesValue,
)
from .health import HealthCheckResponse, HealthCheckResponseStatus
from .organization_invitations import (
    OrganizationInvitationsAcceptResponse,
    OrganizationInvitationsAcceptResponseRole,
    OrganizationInvitationsCreateRequestRole,
    OrganizationInvitationsCreateResponse,
    OrganizationInvitationsCreateResponseRole,
    OrganizationInvitationsCreateResponseStatus,
    OrganizationInvitationsGetResponse,
    OrganizationInvitationsGetResponseRole,
    OrganizationInvitationsGetResponseStatus,
    OrganizationInvitationsListResponseItem,
    OrganizationInvitationsListResponseItemRole,
    OrganizationInvitationsListResponseItemStatus,
)
from .organization_memberships import (
    OrganizationMembershipsListResponseItem,
    OrganizationMembershipsListResponseItemRole,
    OrganizationMembershipsUpdateRequestRole,
    OrganizationMembershipsUpdateResponse,
    OrganizationMembershipsUpdateResponseRole,
)
from .organizations import (
    OrganizationsCreatePaymentIntentResponse,
    OrganizationsCreateResponse,
    OrganizationsCreateResponseRole,
    OrganizationsGetResponse,
    OrganizationsGetResponseRole,
    OrganizationsListResponseItem,
    OrganizationsListResponseItemRole,
    OrganizationsPreviewSubscriptionChangeRequestTargetPlan,
    OrganizationsPreviewSubscriptionChangeResponse,
    OrganizationsPreviewSubscriptionChangeResponseValidationErrorsItem,
    OrganizationsPreviewSubscriptionChangeResponseValidationErrorsItemResource,
    OrganizationsRouterBalanceResponse,
    OrganizationsSubscriptionResponse,
    OrganizationsSubscriptionResponseCurrentPlan,
    OrganizationsSubscriptionResponsePaymentMethod,
    OrganizationsSubscriptionResponseScheduledChange,
    OrganizationsSubscriptionResponseScheduledChangeTargetPlan,
    OrganizationsUpdateResponse,
    OrganizationsUpdateResponseRole,
    OrganizationsUpdateSubscriptionRequestTargetPlan,
    OrganizationsUpdateSubscriptionResponse,
)
from .project_memberships import (
    ProjectMembershipsCreateRequestRole,
    ProjectMembershipsCreateResponse,
    ProjectMembershipsCreateResponseRole,
    ProjectMembershipsGetResponse,
    ProjectMembershipsGetResponseRole,
    ProjectMembershipsListResponseItem,
    ProjectMembershipsListResponseItemRole,
    ProjectMembershipsUpdateRequestRole,
    ProjectMembershipsUpdateResponse,
    ProjectMembershipsUpdateResponseRole,
)
from .projects import (
    ProjectsCreateResponse,
    ProjectsGetResponse,
    ProjectsListResponseItem,
    ProjectsUpdateResponse,
)
from .tags import (
    TagsCreateResponse,
    TagsGetResponse,
    TagsListResponse,
    TagsListResponseTagsItem,
    TagsUpdateResponse,
)
from .token_cost import TokenCostCalculateRequestUsage, TokenCostCalculateResponse
from .traces import (
    TracesCreateOtelRequestResourceSpansItem,
    TracesCreateOtelRequestResourceSpansItemResource,
    TracesCreateOtelRequestResourceSpansItemResourceAttributesItem,
    TracesCreateOtelRequestResourceSpansItemResourceAttributesItemValue,
    TracesCreateOtelRequestResourceSpansItemResourceAttributesItemValueArrayValue,
    TracesCreateOtelRequestResourceSpansItemResourceAttributesItemValueIntValue,
    TracesCreateOtelRequestResourceSpansItemResourceAttributesItemValueKvlistValue,
    TracesCreateOtelRequestResourceSpansItemResourceAttributesItemValueKvlistValueValuesItem,
    TracesCreateOtelRequestResourceSpansItemScopeSpansItem,
    TracesCreateOtelRequestResourceSpansItemScopeSpansItemScope,
    TracesCreateOtelRequestResourceSpansItemScopeSpansItemScopeAttributesItem,
    TracesCreateOtelRequestResourceSpansItemScopeSpansItemScopeAttributesItemValue,
    TracesCreateOtelRequestResourceSpansItemScopeSpansItemScopeAttributesItemValueArrayValue,
    TracesCreateOtelRequestResourceSpansItemScopeSpansItemScopeAttributesItemValueIntValue,
    TracesCreateOtelRequestResourceSpansItemScopeSpansItemScopeAttributesItemValueKvlistValue,
    TracesCreateOtelRequestResourceSpansItemScopeSpansItemScopeAttributesItemValueKvlistValueValuesItem,
    TracesCreateOtelRequestResourceSpansItemScopeSpansItemSpansItem,
    TracesCreateOtelRequestResourceSpansItemScopeSpansItemSpansItemAttributesItem,
    TracesCreateOtelRequestResourceSpansItemScopeSpansItemSpansItemAttributesItemValue,
    TracesCreateOtelRequestResourceSpansItemScopeSpansItemSpansItemAttributesItemValueArrayValue,
    TracesCreateOtelRequestResourceSpansItemScopeSpansItemSpansItemAttributesItemValueIntValue,
    TracesCreateOtelRequestResourceSpansItemScopeSpansItemSpansItemAttributesItemValueKvlistValue,
    TracesCreateOtelRequestResourceSpansItemScopeSpansItemSpansItemAttributesItemValueKvlistValueValuesItem,
    TracesCreateOtelRequestResourceSpansItemScopeSpansItemSpansItemEndTimeUnixNano,
    TracesCreateOtelRequestResourceSpansItemScopeSpansItemSpansItemStartTimeUnixNano,
    TracesCreateOtelRequestResourceSpansItemScopeSpansItemSpansItemStatus,
    TracesCreateOtelResponse,
    TracesCreateOtelResponsePartialSuccess,
    TracesCreateRequestResourceSpansItem,
    TracesCreateRequestResourceSpansItemResource,
    TracesCreateRequestResourceSpansItemResourceAttributesItem,
    TracesCreateRequestResourceSpansItemResourceAttributesItemValue,
    TracesCreateRequestResourceSpansItemResourceAttributesItemValueArrayValue,
    TracesCreateRequestResourceSpansItemResourceAttributesItemValueIntValue,
    TracesCreateRequestResourceSpansItemResourceAttributesItemValueKvlistValue,
    TracesCreateRequestResourceSpansItemResourceAttributesItemValueKvlistValueValuesItem,
    TracesCreateRequestResourceSpansItemScopeSpansItem,
    TracesCreateRequestResourceSpansItemScopeSpansItemScope,
    TracesCreateRequestResourceSpansItemScopeSpansItemScopeAttributesItem,
    TracesCreateRequestResourceSpansItemScopeSpansItemScopeAttributesItemValue,
    TracesCreateRequestResourceSpansItemScopeSpansItemScopeAttributesItemValueArrayValue,
    TracesCreateRequestResourceSpansItemScopeSpansItemScopeAttributesItemValueIntValue,
    TracesCreateRequestResourceSpansItemScopeSpansItemScopeAttributesItemValueKvlistValue,
    TracesCreateRequestResourceSpansItemScopeSpansItemScopeAttributesItemValueKvlistValueValuesItem,
    TracesCreateRequestResourceSpansItemScopeSpansItemSpansItem,
    TracesCreateRequestResourceSpansItemScopeSpansItemSpansItemAttributesItem,
    TracesCreateRequestResourceSpansItemScopeSpansItemSpansItemAttributesItemValue,
    TracesCreateRequestResourceSpansItemScopeSpansItemSpansItemAttributesItemValueArrayValue,
    TracesCreateRequestResourceSpansItemScopeSpansItemSpansItemAttributesItemValueIntValue,
    TracesCreateRequestResourceSpansItemScopeSpansItemSpansItemAttributesItemValueKvlistValue,
    TracesCreateRequestResourceSpansItemScopeSpansItemSpansItemAttributesItemValueKvlistValueValuesItem,
    TracesCreateRequestResourceSpansItemScopeSpansItemSpansItemEndTimeUnixNano,
    TracesCreateRequestResourceSpansItemScopeSpansItemSpansItemStartTimeUnixNano,
    TracesCreateRequestResourceSpansItemScopeSpansItemSpansItemStatus,
    TracesCreateResponse,
    TracesCreateResponsePartialSuccess,
    TracesGetAnalyticsSummaryResponse,
    TracesGetAnalyticsSummaryResponseTopFunctionsItem,
    TracesGetAnalyticsSummaryResponseTopModelsItem,
    TracesGetTraceDetailByEnvResponse,
    TracesGetTraceDetailByEnvResponseSpansItem,
    TracesGetTraceDetailResponse,
    TracesGetTraceDetailResponseSpansItem,
    TracesListByFunctionHashResponse,
    TracesListByFunctionHashResponseTracesItem,
    TracesSearchByEnvRequestAttributeFiltersItem,
    TracesSearchByEnvRequestAttributeFiltersItemOperator,
    TracesSearchByEnvRequestSortBy,
    TracesSearchByEnvRequestSortOrder,
    TracesSearchByEnvResponse,
    TracesSearchByEnvResponseSpansItem,
    TracesSearchRequestAttributeFiltersItem,
    TracesSearchRequestAttributeFiltersItemOperator,
    TracesSearchRequestSortBy,
    TracesSearchRequestSortOrder,
    TracesSearchResponse,
    TracesSearchResponseSpansItem,
)

__all__ = [
    "AlreadyExistsError",
    "AlreadyExistsErrorTag",
    "AnnotationsCreateRequestLabel",
    "AnnotationsCreateResponse",
    "AnnotationsCreateResponseLabel",
    "AnnotationsGetResponse",
    "AnnotationsGetResponseLabel",
    "AnnotationsListRequestLabel",
    "AnnotationsListResponse",
    "AnnotationsListResponseAnnotationsItem",
    "AnnotationsListResponseAnnotationsItemLabel",
    "AnnotationsUpdateRequestLabel",
    "AnnotationsUpdateResponse",
    "AnnotationsUpdateResponseLabel",
    "ApiKeysCreateResponse",
    "ApiKeysGetResponse",
    "ApiKeysListAllForOrgResponseItem",
    "ApiKeysListResponseItem",
    "AsyncMirascope",
    "BadRequestError",
    "BadRequestErrorBody",
    "BadRequestErrorBody_HttpApiDecodeError",
    "BadRequestErrorBody_ImmutableResourceError",
    "ClickHouseError",
    "ConflictError",
    "DatabaseError",
    "DatabaseErrorTag",
    "Date",
    "EnvironmentsCreateResponse",
    "EnvironmentsGetAnalyticsResponse",
    "EnvironmentsGetAnalyticsResponseTopFunctionsItem",
    "EnvironmentsGetAnalyticsResponseTopModelsItem",
    "EnvironmentsGetResponse",
    "EnvironmentsListResponseItem",
    "EnvironmentsUpdateResponse",
    "ForbiddenError",
    "FunctionsCreateRequestDependenciesValue",
    "FunctionsCreateResponse",
    "FunctionsCreateResponseDependenciesValue",
    "FunctionsFindByHashResponse",
    "FunctionsFindByHashResponseDependenciesValue",
    "FunctionsGetByEnvResponse",
    "FunctionsGetByEnvResponseDependenciesValue",
    "FunctionsGetResponse",
    "FunctionsGetResponseDependenciesValue",
    "FunctionsListByEnvResponse",
    "FunctionsListByEnvResponseFunctionsItem",
    "FunctionsListByEnvResponseFunctionsItemDependenciesValue",
    "FunctionsListResponse",
    "FunctionsListResponseFunctionsItem",
    "FunctionsListResponseFunctionsItemDependenciesValue",
    "HealthCheckResponse",
    "HealthCheckResponseStatus",
    "HttpApiDecodeError",
    "HttpApiDecodeErrorTag",
    "ImmutableResourceError",
    "InternalServerError",
    "InternalServerErrorBody",
    "InternalServerErrorBody_ClickHouseError",
    "InternalServerErrorBody_DatabaseError",
    "Issue",
    "IssueTag",
    "Mirascope",
    "MirascopeEnvironment",
    "NotFoundError",
    "NotFoundErrorBody",
    "NotFoundErrorTag",
    "NumberFromString",
    "OrganizationInvitationsAcceptResponse",
    "OrganizationInvitationsAcceptResponseRole",
    "OrganizationInvitationsCreateRequestRole",
    "OrganizationInvitationsCreateResponse",
    "OrganizationInvitationsCreateResponseRole",
    "OrganizationInvitationsCreateResponseStatus",
    "OrganizationInvitationsGetResponse",
    "OrganizationInvitationsGetResponseRole",
    "OrganizationInvitationsGetResponseStatus",
    "OrganizationInvitationsListResponseItem",
    "OrganizationInvitationsListResponseItemRole",
    "OrganizationInvitationsListResponseItemStatus",
    "OrganizationMembershipsListResponseItem",
    "OrganizationMembershipsListResponseItemRole",
    "OrganizationMembershipsUpdateRequestRole",
    "OrganizationMembershipsUpdateResponse",
    "OrganizationMembershipsUpdateResponseRole",
    "OrganizationsCreatePaymentIntentResponse",
    "OrganizationsCreateResponse",
    "OrganizationsCreateResponseRole",
    "OrganizationsGetResponse",
    "OrganizationsGetResponseRole",
    "OrganizationsListResponseItem",
    "OrganizationsListResponseItemRole",
    "OrganizationsPreviewSubscriptionChangeRequestTargetPlan",
    "OrganizationsPreviewSubscriptionChangeResponse",
    "OrganizationsPreviewSubscriptionChangeResponseValidationErrorsItem",
    "OrganizationsPreviewSubscriptionChangeResponseValidationErrorsItemResource",
    "OrganizationsRouterBalanceResponse",
    "OrganizationsSubscriptionResponse",
    "OrganizationsSubscriptionResponseCurrentPlan",
    "OrganizationsSubscriptionResponsePaymentMethod",
    "OrganizationsSubscriptionResponseScheduledChange",
    "OrganizationsSubscriptionResponseScheduledChangeTargetPlan",
    "OrganizationsUpdateResponse",
    "OrganizationsUpdateResponseRole",
    "OrganizationsUpdateSubscriptionRequestTargetPlan",
    "OrganizationsUpdateSubscriptionResponse",
    "PaymentRequiredError",
    "PermissionDeniedError",
    "PermissionDeniedErrorTag",
    "PlanLimitExceededError",
    "PlanLimitExceededErrorTag",
    "PricingUnavailableError",
    "ProjectMembershipsCreateRequestRole",
    "ProjectMembershipsCreateResponse",
    "ProjectMembershipsCreateResponseRole",
    "ProjectMembershipsGetResponse",
    "ProjectMembershipsGetResponseRole",
    "ProjectMembershipsListResponseItem",
    "ProjectMembershipsListResponseItemRole",
    "ProjectMembershipsUpdateRequestRole",
    "ProjectMembershipsUpdateResponse",
    "ProjectMembershipsUpdateResponseRole",
    "ProjectsCreateResponse",
    "ProjectsGetResponse",
    "ProjectsListResponseItem",
    "ProjectsUpdateResponse",
    "PropertyKey",
    "PropertyKeyKey",
    "PropertyKeyKeyTag",
    "RateLimitError",
    "RateLimitErrorTag",
    "ServiceUnavailableError",
    "ServiceUnavailableErrorBody",
    "ServiceUnavailableErrorTag",
    "StripeError",
    "SubscriptionPastDueError",
    "SubscriptionPastDueErrorTag",
    "TagsCreateResponse",
    "TagsGetResponse",
    "TagsListResponse",
    "TagsListResponseTagsItem",
    "TagsUpdateResponse",
    "TokenCostCalculateRequestUsage",
    "TokenCostCalculateResponse",
    "TooManyRequestsError",
    "TracesCreateOtelRequestResourceSpansItem",
    "TracesCreateOtelRequestResourceSpansItemResource",
    "TracesCreateOtelRequestResourceSpansItemResourceAttributesItem",
    "TracesCreateOtelRequestResourceSpansItemResourceAttributesItemValue",
    "TracesCreateOtelRequestResourceSpansItemResourceAttributesItemValueArrayValue",
    "TracesCreateOtelRequestResourceSpansItemResourceAttributesItemValueIntValue",
    "TracesCreateOtelRequestResourceSpansItemResourceAttributesItemValueKvlistValue",
    "TracesCreateOtelRequestResourceSpansItemResourceAttributesItemValueKvlistValueValuesItem",
    "TracesCreateOtelRequestResourceSpansItemScopeSpansItem",
    "TracesCreateOtelRequestResourceSpansItemScopeSpansItemScope",
    "TracesCreateOtelRequestResourceSpansItemScopeSpansItemScopeAttributesItem",
    "TracesCreateOtelRequestResourceSpansItemScopeSpansItemScopeAttributesItemValue",
    "TracesCreateOtelRequestResourceSpansItemScopeSpansItemScopeAttributesItemValueArrayValue",
    "TracesCreateOtelRequestResourceSpansItemScopeSpansItemScopeAttributesItemValueIntValue",
    "TracesCreateOtelRequestResourceSpansItemScopeSpansItemScopeAttributesItemValueKvlistValue",
    "TracesCreateOtelRequestResourceSpansItemScopeSpansItemScopeAttributesItemValueKvlistValueValuesItem",
    "TracesCreateOtelRequestResourceSpansItemScopeSpansItemSpansItem",
    "TracesCreateOtelRequestResourceSpansItemScopeSpansItemSpansItemAttributesItem",
    "TracesCreateOtelRequestResourceSpansItemScopeSpansItemSpansItemAttributesItemValue",
    "TracesCreateOtelRequestResourceSpansItemScopeSpansItemSpansItemAttributesItemValueArrayValue",
    "TracesCreateOtelRequestResourceSpansItemScopeSpansItemSpansItemAttributesItemValueIntValue",
    "TracesCreateOtelRequestResourceSpansItemScopeSpansItemSpansItemAttributesItemValueKvlistValue",
    "TracesCreateOtelRequestResourceSpansItemScopeSpansItemSpansItemAttributesItemValueKvlistValueValuesItem",
    "TracesCreateOtelRequestResourceSpansItemScopeSpansItemSpansItemEndTimeUnixNano",
    "TracesCreateOtelRequestResourceSpansItemScopeSpansItemSpansItemStartTimeUnixNano",
    "TracesCreateOtelRequestResourceSpansItemScopeSpansItemSpansItemStatus",
    "TracesCreateOtelResponse",
    "TracesCreateOtelResponsePartialSuccess",
    "TracesCreateRequestResourceSpansItem",
    "TracesCreateRequestResourceSpansItemResource",
    "TracesCreateRequestResourceSpansItemResourceAttributesItem",
    "TracesCreateRequestResourceSpansItemResourceAttributesItemValue",
    "TracesCreateRequestResourceSpansItemResourceAttributesItemValueArrayValue",
    "TracesCreateRequestResourceSpansItemResourceAttributesItemValueIntValue",
    "TracesCreateRequestResourceSpansItemResourceAttributesItemValueKvlistValue",
    "TracesCreateRequestResourceSpansItemResourceAttributesItemValueKvlistValueValuesItem",
    "TracesCreateRequestResourceSpansItemScopeSpansItem",
    "TracesCreateRequestResourceSpansItemScopeSpansItemScope",
    "TracesCreateRequestResourceSpansItemScopeSpansItemScopeAttributesItem",
    "TracesCreateRequestResourceSpansItemScopeSpansItemScopeAttributesItemValue",
    "TracesCreateRequestResourceSpansItemScopeSpansItemScopeAttributesItemValueArrayValue",
    "TracesCreateRequestResourceSpansItemScopeSpansItemScopeAttributesItemValueIntValue",
    "TracesCreateRequestResourceSpansItemScopeSpansItemScopeAttributesItemValueKvlistValue",
    "TracesCreateRequestResourceSpansItemScopeSpansItemScopeAttributesItemValueKvlistValueValuesItem",
    "TracesCreateRequestResourceSpansItemScopeSpansItemSpansItem",
    "TracesCreateRequestResourceSpansItemScopeSpansItemSpansItemAttributesItem",
    "TracesCreateRequestResourceSpansItemScopeSpansItemSpansItemAttributesItemValue",
    "TracesCreateRequestResourceSpansItemScopeSpansItemSpansItemAttributesItemValueArrayValue",
    "TracesCreateRequestResourceSpansItemScopeSpansItemSpansItemAttributesItemValueIntValue",
    "TracesCreateRequestResourceSpansItemScopeSpansItemSpansItemAttributesItemValueKvlistValue",
    "TracesCreateRequestResourceSpansItemScopeSpansItemSpansItemAttributesItemValueKvlistValueValuesItem",
    "TracesCreateRequestResourceSpansItemScopeSpansItemSpansItemEndTimeUnixNano",
    "TracesCreateRequestResourceSpansItemScopeSpansItemSpansItemStartTimeUnixNano",
    "TracesCreateRequestResourceSpansItemScopeSpansItemSpansItemStatus",
    "TracesCreateResponse",
    "TracesCreateResponsePartialSuccess",
    "TracesGetAnalyticsSummaryResponse",
    "TracesGetAnalyticsSummaryResponseTopFunctionsItem",
    "TracesGetAnalyticsSummaryResponseTopModelsItem",
    "TracesGetTraceDetailByEnvResponse",
    "TracesGetTraceDetailByEnvResponseSpansItem",
    "TracesGetTraceDetailResponse",
    "TracesGetTraceDetailResponseSpansItem",
    "TracesListByFunctionHashResponse",
    "TracesListByFunctionHashResponseTracesItem",
    "TracesSearchByEnvRequestAttributeFiltersItem",
    "TracesSearchByEnvRequestAttributeFiltersItemOperator",
    "TracesSearchByEnvRequestSortBy",
    "TracesSearchByEnvRequestSortOrder",
    "TracesSearchByEnvResponse",
    "TracesSearchByEnvResponseSpansItem",
    "TracesSearchRequestAttributeFiltersItem",
    "TracesSearchRequestAttributeFiltersItemOperator",
    "TracesSearchRequestSortBy",
    "TracesSearchRequestSortOrder",
    "TracesSearchResponse",
    "TracesSearchResponseSpansItem",
    "UnauthorizedError",
    "UnauthorizedErrorBody",
    "UnauthorizedErrorTag",
    "annotations",
    "api_keys",
    "docs",
    "environments",
    "functions",
    "health",
    "organization_invitations",
    "organization_memberships",
    "organizations",
    "project_memberships",
    "projects",
    "tags",
    "token_cost",
    "traces",
]



from inline_snapshot import snapshot

test_snapshot = snapshot(
    {
        "exception": {
            "type": "FeatureNotSupportedError",
            "args": "('Anthropic provider does not support strict tools. Try the beta provider.',)",
            "feature": "strict tools",
            "model_id": "anthropic/claude-sonnet-4-0",
            "provider_id": "anthropic",
        }
    }
)



"""Web search tool for provider-native web search capabilities."""

from dataclasses import dataclass, field

from .provider_tools import ProviderTool


@dataclass(frozen=True)
class WebSearchTool(ProviderTool):
    """Web search tool that allows the model to search the internet.

    This is a provider tool - the search is executed server-side by the provider,
    not by your code. The model decides when to search based on the prompt,
    and the provider returns search results with citations.

    Supported providers include Anthropic, Google, and OpenAI (when using the Responses API).

    Example:
        ```python
        from mirascope import llm

        @llm.call("anthropic/claude-sonnet-4-5", tools=[llm.WebSearchTool()])
        def search_web(query: str) -> str:
            return f"Search the web for: {query}"

        response = search_web("Who won the 2024 Super Bowl?")
        print(response.text())  # Response includes citations from web search
        ```
    """

    name: str = field(default="web_search", init=False)
    """The tool name. Always "web_search" for this tool type."""



from inline_snapshot import snapshot

from mirascope.llm import (
    AssistantMessage,
    SystemMessage,
    Text,
    UserMessage,
)

test_snapshot = snapshot(
    {
        "response": {
            "provider_id": "anthropic",
            "model_id": "anthropic/claude-haiku-4-5",
            "provider_model_name": "claude-haiku-4-5",
            "params": {},
            "finish_reason": None,
            "usage": {
                "input_tokens": 818,
                "output_tokens": 99,
                "cache_read_tokens": 0,
                "cache_write_tokens": 0,
                "reasoning_tokens": 0,
                "provider_tool_usage": None,
                "raw": "Usage(cache_creation=CacheCreation(ephemeral_1h_input_tokens=0, ephemeral_5m_input_tokens=0), cache_creation_input_tokens=0, cache_read_input_tokens=0, input_tokens=818, output_tokens=99, server_tool_use=None, service_tier='standard')",
                "total_tokens": 917,
            },
            "messages": [
                SystemMessage(
                    content=Text(
                        text="Always respond to the user's query using the __mirascope_formatted_output_tool__ tool for structured output."
                    )
                ),
                UserMessage(content=[Text(text="Please recommend three books to me!")]),
                AssistantMessage(
                    content=[
                        Text(
                            text='{"output": [{"title": "The Great Gatsby", "rating": 7}, {"title": "To Kill a Mockingbird", "rating": 7}, {"title": "1984", "rating": 7}]}'
                        )
                    ],
                    provider_id="anthropic",
                    model_id="anthropic/claude-haiku-4-5",
                    provider_model_name="claude-haiku-4-5",
                    raw_message={
                        "role": "assistant",
                        "content": [
                            {
                                "id": "toolu_01UvuQin1uqfJoHuawUZnfDm",
                                "input": {
                                    "output": [
                                        {"title": "The Great Gatsby", "rating": 7},
                                        {"title": "To Kill a Mockingbird", "rating": 7},
                                        {"title": "1984", "rating": 7},
                                    ]
                                },
                                "name": "__mirascope_formatted_output_tool__",
                                "type": "tool_use",
                            }
                        ],
                    },
                ),
            ],
            "format": {
                "name": "list",
                "description": None,
                "schema": {
                    "$defs": {
                        "Book": {
                            "properties": {
                                "title": {"title": "Title", "type": "string"},
                                "rating": {
                                    "description": "For testing purposes, the rating should be 7",
                                    "title": "Rating",
                                    "type": "integer",
                                },
                            },
                            "required": ["title", "rating"],
                            "title": "Book",
                            "type": "object",
                        }
                    },
                    "description": "Wrapper for primitive type list",
                    "properties": {
                        "output": {
                            "items": {"$ref": "#/$defs/Book"},
                            "title": "Output",
                            "type": "array",
                        }
                    },
                    "required": ["output"],
                    "title": "listOutput",
                    "type": "object",
                },
                "mode": "tool",
                "formatting_instructions": "Always respond to the user's query using the __mirascope_formatted_output_tool__ tool for structured output.",
            },
            "tools": [],
        }
    }
)



"""Utilities for handling primitive types in formatting."""

import inspect
from enum import Enum
from types import UnionType
from typing import (
    Annotated,
    Any,
    Literal,
    Protocol,
    TypeAlias,
    Union,
    cast,
    get_args,
    get_origin,
)
from typing_extensions import TypeIs

from pydantic import create_model

PrimitiveType: TypeAlias = (
    str
    | int
    | float
    | bool
    | bytes
    | list[Any]
    | set[Any]
    | tuple[Any, ...]
    | dict[Any, Any]
)
"""Primitive types that can be used with format parameter.

These types are automatically wrapped in a BaseModel for schema generation,
then unwrapped after validation to return the primitive value.
"""


class PrimitiveWrapperModel(Protocol):
    """Protocol for wrapper models with an output field."""

    output: Any
    model_fields: Any

    def __init__(self, *, output: Any) -> None: ...  # noqa: ANN401

    @classmethod
    def model_json_schema(cls) -> dict[str, Any]: ...

    @classmethod
    def model_validate_json(cls, json_data: str) -> "PrimitiveWrapperModel": ...


def is_primitive_type(
    type_: Any,  # noqa: ANN401
) -> TypeIs[type[PrimitiveType]]:
    """Check if a type is a primitive type that needs wrapping.

    Returns True for:
    - Basic primitives: str, int, float, bool, bytes, list, set, tuple, dict
    - Enum types
    - Generic types with primitive origins: list[Book], dict[str, int]
    - Literal types
    - Union types (including Optional)
    - Annotated types

    Returns False for:
    - BaseModel subclasses (already have model_json_schema)
    - None/NoneType

    Args:
        type_: The type to check

    Returns:
        True if the type is a primitive that needs wrapping

    Example:
        >>> is_primitive_type(str)
        True
        >>> is_primitive_type(list[int])
        True
        >>> from pydantic import BaseModel
        >>> class Book(BaseModel):
        ...     title: str
        >>> is_primitive_type(Book)
        False
    """
    primitive_types: set[type[PrimitiveType]] = {
        str,
        int,
        float,
        bool,
        bytes,
        list,
        set,
        tuple,
        dict,
    }
    special_types: set[Any] = {Annotated, Literal, Union, UnionType}

    return (
        (inspect.isclass(type_) and issubclass(type_, Enum))
        or type_ in primitive_types
        or get_origin(type_) in primitive_types.union(special_types)
    )


def _get_type_name(type_: Any) -> str:  # noqa: ANN401
    """Extract a clean name from a type for use in model naming.

    Handles Annotated types by extracting the underlying type,
    and generates clean names for generic types.

    Args:
        type_: The type to extract a name from

    Returns:
        A clean string suitable for use in a Python class name

    Example:
        >>> _get_type_name(str)
        'str'
        >>> _get_type_name(list[int])
        'list_int_'
    """
    # Import Annotated locally
    # Check if this is an Annotated type
    if get_origin(type_) in {Annotated}:
        # For Annotated types, use the first arg (the actual type)
        return _get_type_name(get_args(type_)[0])

    # If the type has a __name__ attribute, use it
    if hasattr(type_, "__name__"):
        return type_.__name__

    # For complex generics like list[Book], use string representation
    type_str = str(type_)

    # Clean up the string to make it a valid Python identifier
    # Replace brackets and commas with underscores
    clean = (
        type_str.replace("[", "_")
        .replace("]", "_")
        .replace(", ", "_")
        .replace(" ", "")
        .replace("'", "")
        .replace('"', "")
    )

    return clean


def create_wrapper_model(
    primitive_type: Any,  # noqa: ANN401
) -> type[PrimitiveWrapperModel]:
    """Create a wrapper BaseModel for a primitive type.

    The wrapper has a single field called "output" containing the primitive value.
    Uses Pydantic's create_model() to generate the wrapper dynamically.

    Args:
        primitive_type: The primitive type to wrap

    Returns:
        A dynamically created BaseModel with an "output" field

    Example:
        >>> wrapper = create_wrapper_model(str)
        >>> instance = wrapper(output="hello")
        >>> instance.output
        'hello'

        >>> from pydantic import BaseModel
        >>> class Book(BaseModel):
        ...     title: str
        >>> wrapper = create_wrapper_model(list[Book])
        >>> books = [Book(title="Test")]
        >>> instance = wrapper(output=books)
        >>> len(instance.output)
        1
    """
    # Get a clean name for the wrapper class
    type_name = _get_type_name(primitive_type)

    # Create wrapper model with "output" field (required)
    wrapper = create_model(
        f"{type_name}Output",
        __doc__=f"Wrapper for primitive type {type_name}",
        output=(primitive_type, ...),  # ... means required
    )

    return cast(type[PrimitiveWrapperModel], wrapper)



from pydantic import BaseModel

from mirascope import llm


class Book(BaseModel):
    title: str
    author: str


@llm.call("openai/gpt-5", format=Book)
def recommend_book(genre: str):
    return f"Please recommend a book in {genre}."


def main():
    response: llm.Response[Book] = recommend_book("fantasy")
    book: Book = response.parse()
    print(f"{book.title} by {book.author}")


main()



"""Versioned call wrappers for @ops.version decorated @llm.call functions."""

from __future__ import annotations

import logging
from dataclasses import dataclass, field
from typing import Concatenate, Generic, cast, overload
from typing_extensions import TypeIs

from ...llm.calls import AsyncCall, AsyncContextCall, Call, ContextCall
from ...llm.context import Context, DepsT
from ...llm.formatting import FormattableT
from ...llm.responses import (
    AsyncContextResponse,
    AsyncContextStreamResponse,
    AsyncResponse,
    AsyncStreamResponse,
    ContextResponse,
    ContextStreamResponse,
    Response,
    StreamResponse,
)
from ...llm.types import P
from ..exceptions import ClosureComputationError
from .closure import Closure
from .protocols import (
    AsyncFunction,
    R,
    SyncFunction,
)
from .versioned_functions import (
    AsyncVersionedFunction,
    AsyncVersionedResult,
    VersionedFunction,
    VersionedResult,
    VersionInfo,
    _BaseVersionedFunction,
)

logger = logging.getLogger(__name__)


def _compute_closure_from_call(
    call_obj: (
        ContextCall[P, DepsT, FormattableT]
        | AsyncContextCall[P, DepsT, FormattableT]
        | Call[P, FormattableT]
        | AsyncCall[P, FormattableT]
    ),
) -> Closure | None:
    """Compute closure from a Call object's prompt function.

    Extracts the original function from the Call's prompt closure and computes
    a Closure object for version tracking. Returns None if closure computation
    fails.
    """
    fn = call_obj.prompt.fn
    closure = getattr(fn, "__closure__", None)
    original_fn = closure[0].cell_contents if closure and len(closure) > 0 else fn
    try:
        return Closure.from_fn(original_fn)
    except ClosureComputationError as e:
        logger.warning(
            "Failed to build closure for %s; continuing without version registration: %s",
            e.qualified_name,
            e,
        )
        return None


def is_version_call_type(
    fn: (
        SyncFunction[P, R]
        | AsyncFunction[P, R]
        | ContextCall[P, DepsT, FormattableT]
        | AsyncContextCall[P, DepsT, FormattableT]
        | Call[P, FormattableT]
        | AsyncCall[P, FormattableT]
    ),
) -> TypeIs[
    ContextCall[P, DepsT, FormattableT]
    | AsyncContextCall[P, DepsT, FormattableT]
    | Call[P, FormattableT]
    | AsyncCall[P, FormattableT]
]:
    """Check if fn is any of the Call types."""
    return isinstance(fn, Call | AsyncCall | ContextCall | AsyncContextCall)


def wrap_version_call(
    fn: (
        ContextCall[P, DepsT, FormattableT]
        | AsyncContextCall[P, DepsT, FormattableT]
        | Call[P, FormattableT]
        | AsyncCall[P, FormattableT]
    ),
    tags: tuple[str, ...],
    name: str | None,
    metadata: dict[str, str],
) -> (
    VersionedContextCall[P, DepsT, FormattableT]
    | VersionedAsyncContextCall[P, DepsT, FormattableT]
    | VersionedCall[P, FormattableT]
    | VersionedAsyncCall[P, FormattableT]
):
    """Wrap a Call object with the appropriate VersionedCall type."""
    if isinstance(fn, AsyncContextCall):
        return VersionedAsyncContextCall(
            _call=fn, tags=tags, name=name, metadata=metadata
        )
    elif isinstance(fn, ContextCall):
        return VersionedContextCall(_call=fn, tags=tags, name=name, metadata=metadata)
    elif isinstance(fn, AsyncCall):
        return VersionedAsyncCall(_call=fn, tags=tags, name=name, metadata=metadata)
    else:
        return VersionedCall(_call=fn, tags=tags, name=name, metadata=metadata)


@dataclass(kw_only=True)
class _BaseVersionedCall:
    """Base class for versioned call wrappers.

    Provides common fields and methods for all versioned call types,
    reducing code duplication across VersionedCall, VersionedAsyncCall,
    VersionedContextCall, and VersionedAsyncContextCall.
    """

    tags: tuple[str, ...]
    """Tags to be associated with versioned calls."""

    name: str | None = None
    """Optional custom name for the versioned function."""

    metadata: dict[str, str] = field(default_factory=dict)
    """Arbitrary key-value pairs for additional metadata."""

    closure: Closure | None = field(init=False, default=None)
    """Cached closure for the original function."""

    def _compute_closure(
        self,
        wrapped_call: (
            ContextCall[P, DepsT, FormattableT]
            | AsyncContextCall[P, DepsT, FormattableT]
            | Call[P, FormattableT]
            | AsyncCall[P, FormattableT]
        ),
        versioned_call: _BaseVersionedFunction[..., object],
        versioned_stream: _BaseVersionedFunction[..., object],
    ) -> None:
        """Initialize closure from wrapped call and assign to versioned wrappers."""
        self.closure = _compute_closure_from_call(wrapped_call)
        if self.closure is not None:
            versioned_call.closure = self.closure
            versioned_stream.closure = self.closure

    @property
    def version_info(self) -> VersionInfo | None:
        """Returns static version metadata for this versioned call."""
        if self.closure is None:
            return None
        return VersionInfo(
            uuid=None,
            hash=self.closure.hash,
            signature_hash=self.closure.signature_hash,
            name=self.name or self.closure.name,
            description=self.closure.docstring,
            version=_BaseVersionedFunction._compute_version(self.closure.hash),
            tags=self.tags,
            metadata=self.metadata,
        )


@dataclass(kw_only=True)
class VersionedCall(_BaseVersionedCall, Generic[P, FormattableT]):
    """Versioned wrapper for synchronous Call objects.

    When @ops.version decorates an @llm.call, it returns a VersionedCall that wraps
    the call and stream methods with versioning capabilities.

    Example:
        ```python
        @ops.version
        @llm.call("gpt-4o-mini")
        def recommend_book(genre: str):
            return f"Recommend a {genre} book"

        # Returns Response directly (but execution is versioned)
        response = recommend_book("fantasy")
        print(response.content)

        # Same as __call__
        response = recommend_book.call("fantasy")

        # Streaming returns StreamResponse (versioned)
        stream = recommend_book.stream("fantasy")
        for chunk in stream:
            print(chunk)

        # Use wrapped to get VersionedResult[Response] with version info
        result = recommend_book.wrapped("fantasy")
        print(result.result.content)
        print(result.function_uuid)

        # Use wrapped_stream to get VersionedResult[StreamResponse]
        result = recommend_book.wrapped_stream("fantasy")

        # Access version info
        info = recommend_book.version_info
        print(info.version, info.hash)
        ```
    """

    _call: Call[P, FormattableT]
    """The wrapped Call object."""

    call: VersionedFunction[P, Response | Response[FormattableT]] = field(init=False)
    """VersionedFunction wrapping the call method."""

    stream: VersionedFunction[P, StreamResponse | StreamResponse[FormattableT]] = field(
        init=False
    )
    """VersionedFunction wrapping the stream method."""

    def __post_init__(self) -> None:
        """Initialize VersionedFunction wrappers for call and stream methods."""
        self.call = VersionedFunction(
            fn=self._call.call,
            tags=self.tags,
            name=self.name,
            metadata=self.metadata,
        )
        self.stream = VersionedFunction(
            fn=self._call.stream,
            tags=self.tags,
            name=self.name,
            metadata=self.metadata,
        )
        self._compute_closure(self._call, self.call, self.stream)

    @overload
    def __call__(
        self: VersionedCall[P, None], *args: P.args, **kwargs: P.kwargs
    ) -> Response: ...

    @overload
    def __call__(
        self: VersionedCall[P, FormattableT], *args: P.args, **kwargs: P.kwargs
    ) -> Response[FormattableT]: ...

    def __call__(
        self, *args: P.args, **kwargs: P.kwargs
    ) -> Response | Response[FormattableT]:
        """Call the versioned function and return Response directly."""
        return self.call(*args, **kwargs)

    @overload
    def wrapped(
        self: VersionedCall[P, None], *args: P.args, **kwargs: P.kwargs
    ) -> VersionedResult[Response]: ...

    @overload
    def wrapped(
        self: VersionedCall[P, FormattableT], *args: P.args, **kwargs: P.kwargs
    ) -> VersionedResult[Response[FormattableT]]: ...

    def wrapped(
        self, *args: P.args, **kwargs: P.kwargs
    ) -> VersionedResult[Response] | VersionedResult[Response[FormattableT]]:
        """Call the versioned function and return a wrapped Response."""
        return cast(
            "VersionedResult[Response] | VersionedResult[Response[FormattableT]]",
            self.call.wrapped(*args, **kwargs),
        )

    @overload
    def wrapped_stream(
        self: VersionedCall[P, None], *args: P.args, **kwargs: P.kwargs
    ) -> VersionedResult[StreamResponse]: ...

    @overload
    def wrapped_stream(
        self: VersionedCall[P, FormattableT], *args: P.args, **kwargs: P.kwargs
    ) -> VersionedResult[StreamResponse[FormattableT]]: ...

    def wrapped_stream(
        self, *args: P.args, **kwargs: P.kwargs
    ) -> (
        VersionedResult[StreamResponse] | VersionedResult[StreamResponse[FormattableT]]
    ):
        """Stream the versioned function and return a wrapped StreamResponse."""
        return cast(
            "VersionedResult[StreamResponse] | VersionedResult[StreamResponse[FormattableT]]",
            self.stream.wrapped(*args, **kwargs),
        )


@dataclass(kw_only=True)
class VersionedAsyncCall(_BaseVersionedCall, Generic[P, FormattableT]):
    """Versioned wrapper for asynchronous AsyncCall objects.

    Example:
        ```python
        @ops.version
        @llm.call("gpt-4o-mini")
        async def recommend_book(genre: str):
            return f"Recommend a {genre} book"

        # Returns AsyncResponse directly (but execution is versioned)
        response = await recommend_book("fantasy")
        print(response.content)

        # Use wrapped to get AsyncVersionedResult[AsyncResponse] with version info
        result = await recommend_book.wrapped("fantasy")
        print(result.result.content)
        print(result.function_uuid)
        ```
    """

    _call: AsyncCall[P, FormattableT]
    """The wrapped AsyncCall object."""

    call: AsyncVersionedFunction[P, AsyncResponse | AsyncResponse[FormattableT]] = (
        field(init=False)
    )
    """AsyncVersionedFunction wrapping the call method."""

    stream: AsyncVersionedFunction[
        P, AsyncStreamResponse | AsyncStreamResponse[FormattableT]
    ] = field(init=False)
    """AsyncVersionedFunction wrapping the stream method."""

    def __post_init__(self) -> None:
        """Initialize AsyncVersionedFunction wrappers for call and stream methods."""
        self.call = AsyncVersionedFunction(
            fn=self._call.call,
            tags=self.tags,
            name=self.name,
            metadata=self.metadata,
        )
        self.stream = AsyncVersionedFunction(
            fn=self._call.stream,
            tags=self.tags,
            name=self.name,
            metadata=self.metadata,
        )
        self._compute_closure(self._call, self.call, self.stream)

    @overload
    async def __call__(
        self: VersionedAsyncCall[P, None], *args: P.args, **kwargs: P.kwargs
    ) -> AsyncResponse: ...

    @overload
    async def __call__(
        self: VersionedAsyncCall[P, FormattableT], *args: P.args, **kwargs: P.kwargs
    ) -> AsyncResponse[FormattableT]: ...

    async def __call__(
        self, *args: P.args, **kwargs: P.kwargs
    ) -> AsyncResponse | AsyncResponse[FormattableT]:
        """Call the versioned function and return AsyncResponse directly."""
        return await self.call(*args, **kwargs)

    @overload
    async def wrapped(
        self: VersionedAsyncCall[P, None], *args: P.args, **kwargs: P.kwargs
    ) -> AsyncVersionedResult[AsyncResponse]: ...

    @overload
    async def wrapped(
        self: VersionedAsyncCall[P, FormattableT], *args: P.args, **kwargs: P.kwargs
    ) -> AsyncVersionedResult[AsyncResponse[FormattableT]]: ...

    async def wrapped(
        self, *args: P.args, **kwargs: P.kwargs
    ) -> (
        AsyncVersionedResult[AsyncResponse]
        | AsyncVersionedResult[AsyncResponse[FormattableT]]
    ):
        """Call the versioned function and return a wrapped Response."""
        return cast(
            "AsyncVersionedResult[AsyncResponse] | AsyncVersionedResult[AsyncResponse[FormattableT]]",
            await self.call.wrapped(*args, **kwargs),
        )

    @overload
    async def wrapped_stream(
        self: VersionedAsyncCall[P, None], *args: P.args, **kwargs: P.kwargs
    ) -> AsyncVersionedResult[AsyncStreamResponse]: ...

    @overload
    async def wrapped_stream(
        self: VersionedAsyncCall[P, FormattableT], *args: P.args, **kwargs: P.kwargs
    ) -> AsyncVersionedResult[AsyncStreamResponse[FormattableT]]: ...

    async def wrapped_stream(
        self, *args: P.args, **kwargs: P.kwargs
    ) -> (
        AsyncVersionedResult[AsyncStreamResponse]
        | AsyncVersionedResult[AsyncStreamResponse[FormattableT]]
    ):
        """Stream the versioned function and return a wrapped StreamResponse."""
        return cast(
            "AsyncVersionedResult[AsyncStreamResponse] | AsyncVersionedResult[AsyncStreamResponse[FormattableT]]",
            await self.stream.wrapped(*args, **kwargs),
        )


@dataclass(kw_only=True)
class VersionedContextCall(_BaseVersionedCall, Generic[P, DepsT, FormattableT]):
    """Versioned wrapper for synchronous ContextCall objects.

    Example:
        ```python
        @ops.version
        @llm.call("gpt-4o-mini")
        def recommend_book(ctx: llm.Context[str], genre: str):
            return f"{ctx.deps} Recommend a {genre} book"

        ctx = llm.Context(deps="As a librarian,")

        # Returns ContextResponse directly (but execution is versioned)
        response = recommend_book(ctx, "fantasy")
        print(response.content)

        # Use wrapped to get VersionedResult[ContextResponse] with version info
        result = recommend_book.wrapped(ctx, "fantasy")
        print(result.result.content)
        ```
    """

    _call: ContextCall[P, DepsT, FormattableT]
    """The wrapped ContextCall object."""

    _call_versioned: VersionedFunction[
        Concatenate[Context[DepsT], P],
        ContextResponse[DepsT, None] | ContextResponse[DepsT, FormattableT],
    ] = field(init=False)
    """VersionedFunction wrapping the call method."""

    _stream_versioned: VersionedFunction[
        Concatenate[Context[DepsT], P],
        ContextStreamResponse[DepsT, None] | ContextStreamResponse[DepsT, FormattableT],
    ] = field(init=False)
    """VersionedFunction wrapping the stream method."""

    def __post_init__(self) -> None:
        """Initialize VersionedFunction wrappers for call and stream methods."""
        self._call_versioned = VersionedFunction(
            fn=self._call.call,
            tags=self.tags,
            name=self.name,
            metadata=self.metadata,
        )
        self._stream_versioned = VersionedFunction(
            fn=self._call.stream,
            tags=self.tags,
            name=self.name,
            metadata=self.metadata,
        )
        self._compute_closure(self._call, self._call_versioned, self._stream_versioned)

    @overload
    def call(
        self: VersionedContextCall[P, DepsT, None],
        ctx: Context[DepsT],
        *args: P.args,
        **kwargs: P.kwargs,
    ) -> ContextResponse[DepsT, None]: ...

    @overload
    def call(
        self: VersionedContextCall[P, DepsT, FormattableT],
        ctx: Context[DepsT],
        *args: P.args,
        **kwargs: P.kwargs,
    ) -> ContextResponse[DepsT, FormattableT]: ...

    def call(
        self, ctx: Context[DepsT], *args: P.args, **kwargs: P.kwargs
    ) -> ContextResponse[DepsT, None] | ContextResponse[DepsT, FormattableT]:
        """Call the versioned function and return ContextResponse directly."""
        return self._call_versioned(ctx, *args, **kwargs)

    @overload
    def __call__(
        self: VersionedContextCall[P, DepsT, None],
        ctx: Context[DepsT],
        *args: P.args,
        **kwargs: P.kwargs,
    ) -> ContextResponse[DepsT, None]: ...

    @overload
    def __call__(
        self: VersionedContextCall[P, DepsT, FormattableT],
        ctx: Context[DepsT],
        *args: P.args,
        **kwargs: P.kwargs,
    ) -> ContextResponse[DepsT, FormattableT]: ...

    def __call__(
        self, ctx: Context[DepsT], *args: P.args, **kwargs: P.kwargs
    ) -> ContextResponse[DepsT, None] | ContextResponse[DepsT, FormattableT]:
        """Call the versioned function and return ContextResponse directly."""
        return self.call(ctx, *args, **kwargs)

    @overload
    def stream(
        self: VersionedContextCall[P, DepsT, None],
        ctx: Context[DepsT],
        *args: P.args,
        **kwargs: P.kwargs,
    ) -> ContextStreamResponse[DepsT, None]: ...

    @overload
    def stream(
        self: VersionedContextCall[P, DepsT, FormattableT],
        ctx: Context[DepsT],
        *args: P.args,
        **kwargs: P.kwargs,
    ) -> ContextStreamResponse[DepsT, FormattableT]: ...

    def stream(
        self, ctx: Context[DepsT], *args: P.args, **kwargs: P.kwargs
    ) -> (
        ContextStreamResponse[DepsT, None] | ContextStreamResponse[DepsT, FormattableT]
    ):
        """Stream the versioned function and return a ContextStreamResponse."""
        return self._stream_versioned(ctx, *args, **kwargs)

    @overload
    def wrapped(
        self: VersionedContextCall[P, DepsT, None],
        ctx: Context[DepsT],
        *args: P.args,
        **kwargs: P.kwargs,
    ) -> VersionedResult[ContextResponse[DepsT, None]]: ...

    @overload
    def wrapped(
        self: VersionedContextCall[P, DepsT, FormattableT],
        ctx: Context[DepsT],
        *args: P.args,
        **kwargs: P.kwargs,
    ) -> VersionedResult[ContextResponse[DepsT, FormattableT]]: ...

    def wrapped(
        self, ctx: Context[DepsT], *args: P.args, **kwargs: P.kwargs
    ) -> VersionedResult[
        ContextResponse[DepsT, None] | ContextResponse[DepsT, FormattableT]
    ]:
        """Call the versioned function and return a wrapped Response."""
        return self._call_versioned.wrapped(ctx, *args, **kwargs)

    @overload
    def wrapped_stream(
        self: VersionedContextCall[P, DepsT, None],
        ctx: Context[DepsT],
        *args: P.args,
        **kwargs: P.kwargs,
    ) -> VersionedResult[ContextStreamResponse[DepsT, None]]: ...

    @overload
    def wrapped_stream(
        self: VersionedContextCall[P, DepsT, FormattableT],
        ctx: Context[DepsT],
        *args: P.args,
        **kwargs: P.kwargs,
    ) -> VersionedResult[ContextStreamResponse[DepsT, FormattableT]]: ...

    def wrapped_stream(
        self, ctx: Context[DepsT], *args: P.args, **kwargs: P.kwargs
    ) -> VersionedResult[
        ContextStreamResponse[DepsT, None] | ContextStreamResponse[DepsT, FormattableT]
    ]:
        """Stream the versioned function and return a wrapped StreamResponse."""
        return self._stream_versioned.wrapped(ctx, *args, **kwargs)


@dataclass(kw_only=True)
class VersionedAsyncContextCall(_BaseVersionedCall, Generic[P, DepsT, FormattableT]):
    """Versioned wrapper for asynchronous AsyncContextCall objects.

    Example:
        ```python
        @ops.version
        @llm.call("gpt-4o-mini")
        async def recommend_book(ctx: llm.Context[str], genre: str):
            return f"{ctx.deps} Recommend a {genre} book"

        ctx = llm.Context(deps="As a librarian,")

        # Returns AsyncContextResponse directly (but execution is versioned)
        response = await recommend_book(ctx, "fantasy")
        print(response.content)

        # Use wrapped to get AsyncVersionedResult[AsyncContextResponse] with version info
        result = await recommend_book.wrapped(ctx, "fantasy")
        print(result.result.content)
        ```
    """

    _call: AsyncContextCall[P, DepsT, FormattableT]
    """The wrapped AsyncContextCall object."""

    _call_versioned: AsyncVersionedFunction[
        Concatenate[Context[DepsT], P],
        AsyncContextResponse[DepsT, None] | AsyncContextResponse[DepsT, FormattableT],
    ] = field(init=False)
    """AsyncVersionedFunction wrapping the call method."""

    _stream_versioned: AsyncVersionedFunction[
        Concatenate[Context[DepsT], P],
        AsyncContextStreamResponse[DepsT, None]
        | AsyncContextStreamResponse[DepsT, FormattableT],
    ] = field(init=False)
    """AsyncVersionedFunction wrapping the stream method."""

    def __post_init__(self) -> None:
        """Initialize AsyncVersionedFunction wrappers for call and stream methods."""
        self._call_versioned = AsyncVersionedFunction(
            fn=self._call.call,
            tags=self.tags,
            name=self.name,
            metadata=self.metadata,
        )
        self._stream_versioned = AsyncVersionedFunction(
            fn=self._call.stream,
            tags=self.tags,
            name=self.name,
            metadata=self.metadata,
        )
        self._compute_closure(self._call, self._call_versioned, self._stream_versioned)

    @overload
    async def call(
        self: VersionedAsyncContextCall[P, DepsT, None],
        ctx: Context[DepsT],
        *args: P.args,
        **kwargs: P.kwargs,
    ) -> AsyncContextResponse[DepsT, None]: ...

    @overload
    async def call(
        self: VersionedAsyncContextCall[P, DepsT, FormattableT],
        ctx: Context[DepsT],
        *args: P.args,
        **kwargs: P.kwargs,
    ) -> AsyncContextResponse[DepsT, FormattableT]: ...

    async def call(
        self, ctx: Context[DepsT], *args: P.args, **kwargs: P.kwargs
    ) -> AsyncContextResponse[DepsT, None] | AsyncContextResponse[DepsT, FormattableT]:
        """Call the versioned function and return AsyncContextResponse directly."""
        return await self._call_versioned(ctx, *args, **kwargs)

    @overload
    async def __call__(
        self: VersionedAsyncContextCall[P, DepsT, None],
        ctx: Context[DepsT],
        *args: P.args,
        **kwargs: P.kwargs,
    ) -> AsyncContextResponse[DepsT, None]: ...

    @overload
    async def __call__(
        self: VersionedAsyncContextCall[P, DepsT, FormattableT],
        ctx: Context[DepsT],
        *args: P.args,
        **kwargs: P.kwargs,
    ) -> AsyncContextResponse[DepsT, FormattableT]: ...

    async def __call__(
        self, ctx: Context[DepsT], *args: P.args, **kwargs: P.kwargs
    ) -> AsyncContextResponse[DepsT, None] | AsyncContextResponse[DepsT, FormattableT]:
        """Call the versioned function and return AsyncContextResponse directly."""
        return await self.call(ctx, *args, **kwargs)

    @overload
    async def stream(
        self: VersionedAsyncContextCall[P, DepsT, None],
        ctx: Context[DepsT],
        *args: P.args,
        **kwargs: P.kwargs,
    ) -> AsyncContextStreamResponse[DepsT, None]: ...

    @overload
    async def stream(
        self: VersionedAsyncContextCall[P, DepsT, FormattableT],
        ctx: Context[DepsT],
        *args: P.args,
        **kwargs: P.kwargs,
    ) -> AsyncContextStreamResponse[DepsT, FormattableT]: ...

    async def stream(
        self, ctx: Context[DepsT], *args: P.args, **kwargs: P.kwargs
    ) -> (
        AsyncContextStreamResponse[DepsT, None]
        | AsyncContextStreamResponse[DepsT, FormattableT]
    ):
        """Stream the versioned function and return an AsyncContextStreamResponse."""
        return await self._stream_versioned(ctx, *args, **kwargs)

    @overload
    async def wrapped(
        self: VersionedAsyncContextCall[P, DepsT, None],
        ctx: Context[DepsT],
        *args: P.args,
        **kwargs: P.kwargs,
    ) -> AsyncVersionedResult[AsyncContextResponse[DepsT, None]]: ...

    @overload
    async def wrapped(
        self: VersionedAsyncContextCall[P, DepsT, FormattableT],
        ctx: Context[DepsT],
        *args: P.args,
        **kwargs: P.kwargs,
    ) -> AsyncVersionedResult[AsyncContextResponse[DepsT, FormattableT]]: ...

    async def wrapped(
        self, ctx: Context[DepsT], *args: P.args, **kwargs: P.kwargs
    ) -> AsyncVersionedResult[
        AsyncContextResponse[DepsT, None] | AsyncContextResponse[DepsT, FormattableT]
    ]:
        """Call the versioned function and return a wrapped Response."""
        return await self._call_versioned.wrapped(ctx, *args, **kwargs)

    @overload
    async def wrapped_stream(
        self: VersionedAsyncContextCall[P, DepsT, None],
        ctx: Context[DepsT],
        *args: P.args,
        **kwargs: P.kwargs,
    ) -> AsyncVersionedResult[AsyncContextStreamResponse[DepsT, None]]: ...

    @overload
    async def wrapped_stream(
        self: VersionedAsyncContextCall[P, DepsT, FormattableT],
        ctx: Context[DepsT],
        *args: P.args,
        **kwargs: P.kwargs,
    ) -> AsyncVersionedResult[AsyncContextStreamResponse[DepsT, FormattableT]]: ...

    async def wrapped_stream(
        self, ctx: Context[DepsT], *args: P.args, **kwargs: P.kwargs
    ) -> AsyncVersionedResult[
        AsyncContextStreamResponse[DepsT, None]
        | AsyncContextStreamResponse[DepsT, FormattableT]
    ]:
        """Stream the versioned function and return a wrapped StreamResponse."""
        return await self._stream_versioned.wrapped(ctx, *args, **kwargs)



"""Model feature testing framework.

This module provides a reusable framework for discovering models from provider APIs
and testing their feature support incrementally.

The core framework lives in the `core` submodule. Provider-specific implementations
are in top-level scripts like `openai.py`, `google.py`, etc.

Usage:
    # Run OpenAI tests
    python -m model_features.openai --list-features

    # Run other provider tests (when implemented)
    python -m model_features.google --list-features
    python -m model_features.anthropic --list-features
"""

from .core import (
    ClientT,
    FeatureTest,
    FeatureTestResult,
    FeatureTestRunner,
    ModelFeatureMatrix,
    ModelFeatureRegistry,
    ModelInfo,
    TestStatus,
)

__all__ = [
    "ClientT",
    "FeatureTest",
    "FeatureTestResult",
    "FeatureTestRunner",
    "ModelFeatureMatrix",
    "ModelFeatureRegistry",
    "ModelInfo",
    "TestStatus",
]



"""Tests for RetryResponse and AsyncRetryResponse property delegation."""

import pytest

from mirascope import llm

from .conftest import CONNECTION_ERROR, RATE_LIMIT_ERROR
from .mock_provider import MockProvider

# --- Sync RetryResponse tests ---


def test_response_properties_delegate_to_wrapped(mock_provider: MockProvider) -> None:
    """Test that all response properties delegate to the wrapped response."""

    @llm.retry(max_retries=1)
    @llm.call("mock/test-model")
    def greet() -> str:
        return "Hello"

    response = greet()

    # Test all delegated properties - these delegate to wrapped_response
    assert response.raw is not None
    assert response.provider_id == "mock"
    assert response.provider_model_name == "test-model"
    assert response.params is not None
    assert response.toolkit is not None
    assert response.messages is not None
    assert response.content is not None
    assert response.texts is not None
    assert response.tool_calls is not None
    assert response.thoughts is not None
    # These may be None depending on mock implementation
    _ = response.finish_reason
    _ = response.usage
    assert response.format is None  # No format specified
    assert response.parse() is None  # No format specified


def test_retry_config_property(mock_provider: MockProvider) -> None:  # noqa: ARG001
    """Test that retry_config property returns the retry configuration."""

    @llm.retry(max_retries=3)
    @llm.call("mock/test-model")
    def greet() -> str:
        return "Hello"

    response = greet()

    assert response.retry_config.max_retries == 3


@pytest.mark.asyncio
async def test_async_retry_config_property(mock_provider: MockProvider) -> None:  # noqa: ARG001
    """Test that async retry_config property returns the retry configuration."""

    @llm.retry(max_retries=3)
    @llm.call("mock/test-model")
    async def greet() -> str:
        return "Hello"

    response = await greet()

    assert response.retry_config.max_retries == 3


def test_execute_tools(mock_provider: MockProvider) -> None:  # noqa: ARG001
    """Test that execute_tools delegates to wrapped response."""

    @llm.retry(max_retries=1)
    @llm.call("mock/test-model")
    def greet() -> str:
        return "Hello"

    response = greet()
    result = response.execute_tools()
    assert result == []  # No tool calls in mock response


# --- Async RetryResponse tests ---


@pytest.mark.asyncio
async def test_async_execute_tools(mock_provider: MockProvider) -> None:  # noqa: ARG001
    """Test that async execute_tools delegates to wrapped response."""

    @llm.retry(max_retries=1)
    @llm.call("mock/test-model")
    async def greet() -> str:
        return "Hello"

    response = await greet()
    result = await response.execute_tools()
    assert result == []  # No tool calls in mock response


# --- Resume with fallback tests ---


def test_resume_uses_fallback_model_when_original_succeeded_on_fallback(
    mock_provider: MockProvider,
) -> None:
    """Test that resume uses the fallback model when original response succeeded on fallback."""
    # First call: primary fails, fallback succeeds
    mock_provider.set_exceptions([CONNECTION_ERROR])

    retry_model = llm.retry_model(
        "mock/primary", max_retries=0, fallback_models=["mock/fallback"]
    )

    response = retry_model.call("Hello")

    # Response should have succeeded on fallback
    assert response.model_id == "mock/fallback"
    assert len(response.retry_failures) == 1

    # Resume should use the fallback model (which succeeded)
    resumed = response.resume("Follow up")

    # Should succeed on fallback (no errors set)
    assert resumed.model_id == "mock/fallback"


def test_resume_can_fall_back_to_original_if_fallback_fails(
    mock_provider: MockProvider,
) -> None:
    """Test that resume can fall back to original model if the fallback model fails."""
    # First call: primary fails, fallback succeeds
    mock_provider.set_exceptions([CONNECTION_ERROR])

    retry_model = llm.retry_model(
        "mock/primary", max_retries=0, fallback_models=["mock/fallback"]
    )

    response = retry_model.call("Hello")

    # Response should have succeeded on fallback
    assert response.model_id == "mock/fallback"

    # For resume: fallback fails, primary succeeds
    mock_provider.set_exceptions([RATE_LIMIT_ERROR])

    resumed = response.resume("Follow up")

    # Should have fallen back to primary and succeeded
    assert resumed.model_id == "mock/primary"
    assert len(resumed.retry_failures) == 1
    assert resumed.retry_failures[0].model.model_id == "mock/fallback"


@pytest.mark.asyncio
async def test_async_resume_uses_fallback_model_when_original_succeeded_on_fallback(
    mock_provider: MockProvider,
) -> None:
    """Test that async resume uses the fallback model when original response succeeded on fallback."""
    # First call: primary fails, fallback succeeds
    mock_provider.set_exceptions([CONNECTION_ERROR])

    retry_model = llm.retry_model(
        "mock/primary", max_retries=0, fallback_models=["mock/fallback"]
    )

    response = await retry_model.call_async("Hello")

    # Response should have succeeded on fallback
    assert response.model_id == "mock/fallback"
    assert len(response.retry_failures) == 1

    # Resume should use the fallback model (which succeeded)
    resumed = await response.resume("Follow up")

    # Should succeed on fallback (no errors set)
    assert resumed.model_id == "mock/fallback"


@pytest.mark.asyncio
async def test_async_resume_can_fall_back_to_original_if_fallback_fails(
    mock_provider: MockProvider,
) -> None:
    """Test that async resume can fall back to original model if the fallback model fails."""
    # First call: primary fails, fallback succeeds
    mock_provider.set_exceptions([CONNECTION_ERROR])

    retry_model = llm.retry_model(
        "mock/primary", max_retries=0, fallback_models=["mock/fallback"]
    )

    response = await retry_model.call_async("Hello")

    # Response should have succeeded on fallback
    assert response.model_id == "mock/fallback"

    # For resume: fallback fails, primary succeeds
    mock_provider.set_exceptions([RATE_LIMIT_ERROR])

    resumed = await response.resume("Follow up")

    # Should have fallen back to primary and succeeded
    assert resumed.model_id == "mock/primary"
    assert len(resumed.retry_failures) == 1
    assert resumed.retry_failures[0].model.model_id == "mock/fallback"


# --- Context retry response tests ---


def test_context_retry_config_property(mock_provider: MockProvider) -> None:  # noqa: ARG001
    """Test that ContextRetryResponse.retry_config property returns the retry configuration."""
    retry_model = llm.retry_model("mock/test-model", max_retries=3)
    ctx = llm.Context(deps=None)

    response = retry_model.context_call("Hello", ctx=ctx)

    assert response.retry_config.max_retries == 3


@pytest.mark.asyncio
async def test_async_context_retry_config_property(
    mock_provider: MockProvider,  # noqa: ARG001
) -> None:
    """Test that AsyncContextRetryResponse.retry_config property returns the retry configuration."""
    retry_model = llm.retry_model("mock/test-model", max_retries=3)
    ctx = llm.Context(deps=None)

    response = await retry_model.context_call_async("Hello", ctx=ctx)

    assert response.retry_config.max_retries == 3



from inline_snapshot import snapshot

from mirascope.llm import (
    AssistantMessage,
    Text,
    UserMessage,
)

sync_snapshot = snapshot(
    {
        "response": {
            "provider_id": "anthropic",
            "model_id": "anthropic/claude-sonnet-4-0",
            "provider_model_name": "claude-sonnet-4-0",
            "params": {"thinking": {"level": "medium", "include_thoughts": False}},
            "finish_reason": None,
            "usage": {
                "input_tokens": 67,
                "output_tokens": 933,
                "cache_read_tokens": 0,
                "cache_write_tokens": 0,
                "reasoning_tokens": 0,
                "provider_tool_usage": None,
                "raw": "Usage(cache_creation=CacheCreation(ephemeral_1h_input_tokens=0, ephemeral_5m_input_tokens=0), cache_creation_input_tokens=0, cache_read_input_tokens=0, input_tokens=67, output_tokens=933, server_tool_use=None, service_tier='standard')",
                "total_tokens": 1000,
            },
            "messages": [
                UserMessage(
                    content=[
                        Text(
                            text="How many primes below 400 contain 79 as a substring? Answer ONLY with the number, not sharing which primes they are."
                        )
                    ]
                ),
                AssistantMessage(
                    content=[Text(text="3")],
                    provider_id="anthropic",
                    model_id="anthropic/claude-sonnet-4-0",
                    provider_model_name="claude-sonnet-4-0",
                    raw_message={
                        "role": "assistant",
                        "content": [
                            {
                                "signature": "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",
                                "thinking": """\
I need to find all primes below 400 that contain "79" as a substring.

First, let me think about what numbers below 400 could contain "79" as a substring:
- Numbers of the form 79X (where X is any digit): 790, 791, 792, 793, 794, 795, 796, 797, 798, 799
- Numbers of the form X79 (where X is any digit): 179, 279, 379
- The number 79 itself

Wait, let me be more systematic. Since we're looking at numbers below 400, the possibilities are:
- 79 (the number itself)
- 179, 279, 379 (3-digit numbers ending in 79)
- Numbers starting with 79 but since we need to be below 400, we can't have any 4-digit numbers starting with 79

Let me check each of these for primality:

1) 79: Let me check if 79 is prime.
   79 = ? \n\
   Need to check divisibility by primes up to √79 ≈ 8.9
   So check 2, 3, 5, 7
   79 is odd, so not divisible by 2
   7+9 = 16, not divisible by 3
   Doesn't end in 0 or 5, so not divisible by 5
   79 ÷ 7 = 11.28..., so not divisible by 7
   So 79 is prime.

2) 179: Let me check if 179 is prime.
   Need to check divisibility by primes up to √179 ≈ 13.4
   So check 2, 3, 5, 7, 11, 13
   179 is odd, so not divisible by 2
   1+7+9 = 17, not divisible by 3
   Testing 5, 7, 11, and 13 all yield non-integer results, so 179 is prime.

3) 279: The digit sum is 2+7+9 = 18, which is divisible by 3, so 279 = 3 × 93 and is composite.

4) 379: Checking divisibility by primes up to √379 ≈ 19.5. It's odd, so not divisible by 2. The digit sum 3+7+9 = 19 isn't divisible by 3. It doesn't end in 0 or 5. Testing 7, 11, and 13 all produce non-integers, so 379 appears to be prime based on these checks.

Continuing with 17 and 19: 379 ÷ 17 = 22.29... and 379 ÷ 19 = 19.95..., confirming 379 is prime.

The primes below 400 containing "79" as a substring are 79, 179, and 379—three primes total.\
""",
                                "type": "thinking",
                            },
                            {"citations": None, "text": "3", "type": "text"},
                        ],
                    },
                ),
            ],
            "format": None,
            "tools": [],
        }
    }
)
async_snapshot = snapshot(
    {
        "response": {
            "provider_id": "anthropic",
            "model_id": "anthropic/claude-sonnet-4-0",
            "provider_model_name": "claude-sonnet-4-0",
            "params": {"thinking": {"level": "medium", "include_thoughts": False}},
            "finish_reason": None,
            "usage": {
                "input_tokens": 67,
                "output_tokens": 1215,
                "cache_read_tokens": 0,
                "cache_write_tokens": 0,
                "reasoning_tokens": 0,
                "provider_tool_usage": None,
                "raw": "Usage(cache_creation=CacheCreation(ephemeral_1h_input_tokens=0, ephemeral_5m_input_tokens=0), cache_creation_input_tokens=0, cache_read_input_tokens=0, input_tokens=67, output_tokens=1215, server_tool_use=None, service_tier='standard')",
                "total_tokens": 1282,
            },
            "messages": [
                UserMessage(
                    content=[
                        Text(
                            text="How many primes below 400 contain 79 as a substring? Answer ONLY with the number, not sharing which primes they are."
                        )
                    ]
                ),
                AssistantMessage(
                    content=[
                        Text(
                            text="""\
Looking for primes below 400 that contain "79" as a substring.

The candidates are: 79, 179, 279, 379

Checking primality:
- 79: prime
- 179: prime  \n\
- 279: divisible by 3 (2+7+9=18), so not prime
- 379: prime

3\
"""
                        )
                    ],
                    provider_id="anthropic",
                    model_id="anthropic/claude-sonnet-4-0",
                    provider_model_name="claude-sonnet-4-0",
                    raw_message={
                        "role": "assistant",
                        "content": [
                            {
                                "signature": "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",
                                "thinking": """\
I need to find all prime numbers below 400 that contain "79" as a substring.

Let me think about this systematically. A number contains "79" as a substring if "79" appears consecutively in its decimal representation.

First, let me list numbers that contain "79" as a substring and are below 400:
- 79 itself
- 179
- 279
- 379
- 790, 791, 792, etc. - but these are all ≥ 400, so not relevant
- Numbers like 1790, 2790, etc. - also too large

Wait, I also need to consider numbers where 79 appears in the middle or at the end:
- Numbers of the form X79 where X is a single digit: 079 (which is just 79), 179, 279, 379
- Numbers of the form 79X where X is a digit: 790, 791, 792, 793, 794, 795, 796, 797, 798, 799 - but these are all ≥ 400
- Numbers of the form X79Y where X and Y are digits - but the smallest would be 1790, which is > 400

So the candidates are: 79, 179, 279, 379

Now I need to check which of these are prime:

79: Let me check if 79 is prime.
- Is 79 divisible by 2? No (it's odd)
- Is 79 divisible by 3? 7+9 = 16, 16 is not divisible by 3, so no
- Is 79 divisible by 5? No (doesn't end in 0 or 5)
- Is 79 divisible by 7? 79 ÷ 7 = 11.28..., so no
- Is 79 divisible by 11? 79 ÷ 11 = 7.18..., so no


- Since √79 < 9, I only need to check primes up to 7, so 79 is prime.

179: Let me check if 179 is prime. Testing divisibility by 2, 3, 5, and 7 shows none divide evenly into 179. Since √179 ≈ 13.4, I need to also check 11 and 13, but 179 ÷ 11 = 16.27... and 179 ÷ 13 = 13.77..., so 179 is prime.

279: Checking divisibility by 2 and 3 - the digit sum is 18, which is divisible by 3, so 279 ÷ 3 = 93. This means 279 = 3 × 93, so it's not prime.

379: Let me check if 379 is prime. It's odd, so not divisible by 2. The digit sum is 19, which isn't divisible by 3. Testing 5, 7, 11, 13, and 17 all yield non-integer results, and since √379 ≈ 19.5, I need to continue checking a few more factors to confirm it's prime.

I've verified 379 isn't divisible by 19 either, which completes the necessary checks. The primes below 400 containing "79" are 79, 179, and 379—three total.\
""",
                                "type": "thinking",
                            },
                            {
                                "citations": None,
                                "text": """\
Looking for primes below 400 that contain "79" as a substring.

The candidates are: 79, 179, 279, 379

Checking primality:
- 79: prime
- 179: prime  \n\
- 279: divisible by 3 (2+7+9=18), so not prime
- 379: prime

3\
""",
                                "type": "text",
                            },
                        ],
                    },
                ),
            ],
            "format": None,
            "tools": [],
        }
    }
)
stream_snapshot = snapshot(
    {
        "response": {
            "provider_id": "anthropic",
            "model_id": "anthropic/claude-sonnet-4-0",
            "provider_model_name": "claude-sonnet-4-0",
            "finish_reason": None,
            "messages": [
                UserMessage(
                    content=[
                        Text(
                            text="How many primes below 400 contain 79 as a substring? Answer ONLY with the number, not sharing which primes they are."
                        )
                    ]
                ),
                AssistantMessage(
                    content=[Text(text="3")],
                    provider_id="anthropic",
                    model_id="anthropic/claude-sonnet-4-0",
                    provider_model_name="claude-sonnet-4-0",
                    raw_message={
                        "role": "assistant",
                        "content": [
                            {
                                "type": "thinking",
                                "thinking": """\
I need to find all prime numbers below 400 that contain "79" as a substring.

First, let me think about what numbers below 400 could contain "79" as a substring:
- Numbers of the form 79X (where X is a digit): 790, 791, 792, 793, 794, 795, 796, 797, 798, 799
- Numbers of the form X79 (where X is a digit): 079 (which is just 79), 179, 279, 379
- Numbers of the form X79Y (where X and Y are digits): these would be at least 1790, which is above 400

Wait, let me be more systematic. Numbers below 400 that contain "79":
- 79
- 179
- 279
- 379
- 790, 791, 792, 793, 794, 795, 796, 797, 798, 799

But wait, 790-799 are all ≥ 400, so they don't count.

So I need to check: 79, 179, 279, 379

Let me check if each is prime:

79: Let me check divisibility by small primes.
- Not divisible by 2 (it's odd)
- Not divisible by 3 (7+9=16, not divisible by 3)
- Not divisible by 5 (doesn't end in 0 or 5)
- Not divisible by 7: 79 ÷ 7 = 11.28... not divisible
- √79 ≈ 8.9, so I only need to check primes up to 8
- Already checked 2, 3, 5, 7
So 79 is prime.

179: \n\
- Not divisible by 2 (it's odd)
- Not divisible by 3 (1+7+9=17, not divisible by 3)


- Not divisible by 5, 7, 11, or 13 based on division tests
- Since √179 ≈ 13.4, I've checked all necessary primes
So 179 is prime.

For 279, the digit sum is 18, which is divisible by 3, so 279 = 3 × 93 and isn't prime. Moving to 379: it's odd, the digit sum of 19 rules out divisibility by 3, and it doesn't end in 0 or 5. Testing 7 and 11 both fail, and I'm checking 13 next since √379 ≈ 19.5 means I need to verify primes up to that point.

Continuing the divisibility checks for 379: 13, 17, and 19 all fail to divide it evenly. Since I've now tested every prime up to the square root, 379 is prime. The complete list of numbers below 400 containing "79" as a substring that are actually prime comes to three: 79, 179, and 379.\
""",
                                "signature": "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",
                            },
                            {"type": "text", "text": "3"},
                        ],
                    },
                ),
            ],
            "format": None,
            "tools": [],
            "usage": {
                "input_tokens": 67,
                "output_tokens": 990,
                "cache_read_tokens": 0,
                "cache_write_tokens": 0,
                "reasoning_tokens": 0,
                "provider_tool_usage": None,
                "raw": "None",
                "total_tokens": 1057,
            },
            "n_chunks": 3,
        }
    }
)
async_stream_snapshot = snapshot(
    {
        "response": {
            "provider_id": "anthropic",
            "model_id": "anthropic/claude-sonnet-4-0",
            "provider_model_name": "claude-sonnet-4-0",
            "finish_reason": None,
            "messages": [
                UserMessage(
                    content=[
                        Text(
                            text="How many primes below 400 contain 79 as a substring? Answer ONLY with the number, not sharing which primes they are."
                        )
                    ]
                ),
                AssistantMessage(
                    content=[
                        Text(
                            text="""\
Looking at numbers below 400 that contain "79" as a substring:
- 79
- 179  \n\
- 279
- 379

Checking which are prime:
- 79: prime
- 179: prime
- 279: divisible by 3 (2+7+9=18), so not prime
- 379: prime

3\
"""
                        )
                    ],
                    provider_id="anthropic",
                    model_id="anthropic/claude-sonnet-4-0",
                    provider_model_name="claude-sonnet-4-0",
                    raw_message={
                        "role": "assistant",
                        "content": [
                            {
                                "type": "thinking",
                                "thinking": """\
I need to find all prime numbers below 400 that contain "79" as a substring.

First, let me think about what numbers below 400 could contain "79" as a substring:
- Numbers of the form 79x where x is a single digit: 790, 791, 792, 793, 794, 795, 796, 797, 798, 799
- Numbers of the form x79 where x is a single digit: 179, 279, 379
- Numbers of the form x79y where x and y are single digits, but this would be at least 1000, so too big
- The number 79 itself

Wait, let me be more systematic. Numbers below 400 that contain "79":
- 79
- 179
- 279
- 379
- 790, 791, 792, 793, 794, 795, 796, 797, 798, 799 (but these are all ≥ 790 > 400, so not relevant)

So the candidates are: 79, 179, 279, 379

Now I need to check which of these are prime:

79: Let me check if 79 is prime.
- Not divisible by 2 (it's odd)
- Not divisible by 3 (7+9=16, not divisible by 3)
- Not divisible by 5 (doesn't end in 0 or 5)
- Not divisible by 7 (79 ÷ 7 = 11.28...)
- Since √79 ≈ 8.9, I only need to check up to 8
- Not divisible by 8, but 8 is even so already covered by 2

So 79 is prime.

179: Let me check if 179 is prime.
- Not divisible by 2 (it's odd)
- Not divisible by 3 (1+7+9=17, not divisible by 3)


- Not divisible by 5 (doesn't end in 0 or 5)
- Not divisible by 7 (179 ÷ 7 = 25.57...)
- Not divisible by 11 (179 ÷ 11 = 16.27...)
- Not divisible by 13 (179 ÷ 13 = 13.77...)
- Since √179 ≈ 13.4, I need to check up to 13

So 179 is prime.

279: Let me check if 279 is prime.
- Not divisible by 2 (it's odd)
- Divisible by 3 (2+7+9=18, which is divisible by 3), so 279 = 3 × 93

So 279 is not prime. Now checking 379: it's odd, and the digit sum is 19 (not divisible by 3), doesn't end in 5, and testing divisibility by 7, 11, 13, and 17 shows it's not divisible by any of these. Since √379 ≈ 19.5, 379 is prime. 13 = 29.15...). Since the square root of 379 is approximately 19.5, I only need to check divisibility up to 19, and I've confirmed it's not divisible by any prime up to that point. So 379 is prime.

The numbers below 400 containing "79" as a substring that are prime are 79, 179, and 379—three primes total.\
""",
                                "signature": "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",
                            },
                            {
                                "type": "text",
                                "text": """\
Looking at numbers below 400 that contain "79" as a substring:
- 79
- 179  \n\
- 279
- 379

Checking which are prime:
- 79: prime
- 179: prime
- 279: divisible by 3 (2+7+9=18), so not prime
- 379: prime

3\
""",
                            },
                        ],
                    },
                ),
            ],
            "format": None,
            "tools": [],
            "usage": {
                "input_tokens": 67,
                "output_tokens": 1042,
                "cache_read_tokens": 0,
                "cache_write_tokens": 0,
                "reasoning_tokens": 0,
                "provider_tool_usage": None,
                "raw": "None",
                "total_tokens": 1109,
            },
            "n_chunks": 18,
        }
    }
)



"""End-to-end tests for structured output."""

import inspect

import pytest
from pydantic import BaseModel

from mirascope import llm
from tests.e2e.conftest import FORMATTING_MODES, STRUCTURED_OUTPUT_MODEL_IDS
from tests.utils import (
    Snapshot,
    snapshot_test,
)


class Book(BaseModel):
    title: str
    author: str
    rating: int

    @classmethod
    def formatting_instructions(cls) -> str:
        return inspect.cleandoc("""
        Output a structured book as JSON in the format {title: str, author: str, rating: int}.
        The title should be in all caps, and the rating should always be the
        lucky number 7.
        """)


@pytest.mark.parametrize("model_id", STRUCTURED_OUTPUT_MODEL_IDS)
@pytest.mark.parametrize("formatting_mode", FORMATTING_MODES)
@pytest.mark.vcr
def test_structured_output_with_formatting_instructions(
    model_id: llm.ModelId,
    formatting_mode: llm.FormattingMode | None,
    snapshot: Snapshot,
) -> None:
    """Test structured output without context."""

    format = (
        llm.format(Book, mode=formatting_mode) if formatting_mode is not None else Book
    )

    @llm.call(model_id, format=format)
    def recommend_book(book: str) -> list[llm.Message]:
        return [
            llm.messages.system(f"Always recommend {book}."),
            llm.messages.user("Please recommend a book to me!"),
        ]

    with snapshot_test(snapshot) as snap:
        response = recommend_book("The Name of the Wind")
        snap.set_response(response)

        book = response.parse()
        assert book.author == "Patrick Rothfuss"
        assert book.title == "THE NAME OF THE WIND"
        assert book.rating == 7



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .api_keys_create_response import ApiKeysCreateResponse
from .api_keys_get_response import ApiKeysGetResponse
from .api_keys_list_all_for_org_response_item import ApiKeysListAllForOrgResponseItem
from .api_keys_list_response_item import ApiKeysListResponseItem

__all__ = [
    "ApiKeysCreateResponse",
    "ApiKeysGetResponse",
    "ApiKeysListAllForOrgResponseItem",
    "ApiKeysListResponseItem",
]



from dotenv import load_dotenv

from mirascope import llm

load_dotenv()


@llm.call("openai/gpt-5", thinking={"level": "medium"})
def count_primes() -> str:
    return "How many primes below 200 have 79 as a substring? Answer ONLY with the number of primes, not the primes themselves."


response = count_primes()
print(response.pretty())

with llm.model("openai/gpt-5", thinking={"level": "minimal"}):
    response = response.resume(
        "If you remember the primes, list them. Or say 'I dont remember'"
    )
print(response.pretty())



from inline_snapshot import snapshot

from mirascope.llm import (
    AssistantMessage,
    Text,
    UserMessage,
)

sync_snapshot = snapshot(
    {
        "response": {
            "provider_id": "openai",
            "model_id": "openai/gpt-4o:responses",
            "provider_model_name": "gpt-4o:responses",
            "params": {},
            "finish_reason": None,
            "usage": {
                "input_tokens": 166,
                "output_tokens": 30,
                "cache_read_tokens": 0,
                "cache_write_tokens": 0,
                "reasoning_tokens": 0,
                "provider_tool_usage": None,
                "raw": "ResponseUsage(input_tokens=166, input_tokens_details=InputTokensDetails(cached_tokens=0), output_tokens=30, output_tokens_details=OutputTokensDetails(reasoning_tokens=0), total_tokens=196)",
                "total_tokens": 196,
            },
            "messages": [
                UserMessage(
                    content=[
                        Text(
                            text="Please recommend the most popular book by Patrick Rothfuss"
                        )
                    ]
                ),
                AssistantMessage(
                    content=[
                        Text(
                            text='{"title":"THE NAME OF THE WIND","author":{"first_name":"Patrick","last_name":"Rothfuss"},"rating":7}'
                        )
                    ],
                    provider_id="openai",
                    model_id="openai/gpt-4o:responses",
                    provider_model_name="gpt-4o:responses",
                    raw_message=[
                        {
                            "id": "msg_005e0ab32c3df1050068f966cedb648190a1769c7ec5dea7e9",
                            "content": [
                                {
                                    "annotations": [],
                                    "text": '{"title":"THE NAME OF THE WIND","author":{"first_name":"Patrick","last_name":"Rothfuss"},"rating":7}',
                                    "type": "output_text",
                                    "logprobs": [],
                                }
                            ],
                            "role": "assistant",
                            "status": "completed",
                            "type": "message",
                        }
                    ],
                ),
            ],
            "format": {
                "name": "Book",
                "description": "A book with a rating. The title should be in all caps!",
                "schema": {
                    "$defs": {
                        "Author": {
                            "description": "The author of a book.",
                            "properties": {
                                "first_name": {"title": "First Name", "type": "string"},
                                "last_name": {"title": "Last Name", "type": "string"},
                            },
                            "required": ["first_name", "last_name"],
                            "title": "Author",
                            "type": "object",
                            "additionalProperties": False,
                        }
                    },
                    "description": "A book with a rating. The title should be in all caps!",
                    "properties": {
                        "title": {"title": "Title", "type": "string"},
                        "author": {"$ref": "#/$defs/Author"},
                        "rating": {
                            "description": "For testing purposes, the rating should be 7",
                            "title": "Rating",
                            "type": "integer",
                        },
                    },
                    "required": ["title", "author", "rating"],
                    "title": "Book",
                    "type": "object",
                },
                "mode": "strict",
                "formatting_instructions": None,
            },
            "tools": [],
        }
    }
)
async_snapshot = snapshot(
    {
        "response": {
            "provider_id": "openai",
            "model_id": "openai/gpt-4o:responses",
            "provider_model_name": "gpt-4o:responses",
            "params": {},
            "finish_reason": None,
            "usage": {
                "input_tokens": 166,
                "output_tokens": 30,
                "cache_read_tokens": 0,
                "cache_write_tokens": 0,
                "reasoning_tokens": 0,
                "provider_tool_usage": None,
                "raw": "ResponseUsage(input_tokens=166, input_tokens_details=InputTokensDetails(cached_tokens=0), output_tokens=30, output_tokens_details=OutputTokensDetails(reasoning_tokens=0), total_tokens=196)",
                "total_tokens": 196,
            },
            "messages": [
                UserMessage(
                    content=[
                        Text(
                            text="Please recommend the most popular book by Patrick Rothfuss"
                        )
                    ]
                ),
                AssistantMessage(
                    content=[
                        Text(
                            text='{"title":"THE NAME OF THE WIND","author":{"first_name":"Patrick","last_name":"Rothfuss"},"rating":7}'
                        )
                    ],
                    provider_id="openai",
                    model_id="openai/gpt-4o:responses",
                    provider_model_name="gpt-4o:responses",
                    raw_message=[
                        {
                            "id": "msg_0e2af4b9d9313f560068f966d8ede48197886f72c0b5c845a2",
                            "content": [
                                {
                                    "annotations": [],
                                    "text": '{"title":"THE NAME OF THE WIND","author":{"first_name":"Patrick","last_name":"Rothfuss"},"rating":7}',
                                    "type": "output_text",
                                    "logprobs": [],
                                }
                            ],
                            "role": "assistant",
                            "status": "completed",
                            "type": "message",
                        }
                    ],
                ),
            ],
            "format": {
                "name": "Book",
                "description": "A book with a rating. The title should be in all caps!",
                "schema": {
                    "$defs": {
                        "Author": {
                            "description": "The author of a book.",
                            "properties": {
                                "first_name": {"title": "First Name", "type": "string"},
                                "last_name": {"title": "Last Name", "type": "string"},
                            },
                            "required": ["first_name", "last_name"],
                            "title": "Author",
                            "type": "object",
                            "additionalProperties": False,
                        }
                    },
                    "description": "A book with a rating. The title should be in all caps!",
                    "properties": {
                        "title": {"title": "Title", "type": "string"},
                        "author": {"$ref": "#/$defs/Author"},
                        "rating": {
                            "description": "For testing purposes, the rating should be 7",
                            "title": "Rating",
                            "type": "integer",
                        },
                    },
                    "required": ["title", "author", "rating"],
                    "title": "Book",
                    "type": "object",
                },
                "mode": "strict",
                "formatting_instructions": None,
            },
            "tools": [],
        }
    }
)
stream_snapshot = snapshot(
    {
        "response": {
            "provider_id": "openai",
            "model_id": "openai/gpt-4o:responses",
            "provider_model_name": "gpt-4o:responses",
            "finish_reason": None,
            "messages": [
                UserMessage(
                    content=[
                        Text(
                            text="Please recommend the most popular book by Patrick Rothfuss"
                        )
                    ]
                ),
                AssistantMessage(
                    content=[
                        Text(
                            text='{"title":"THE NAME OF THE WIND","author":{"first_name":"Patrick","last_name":"Rothfuss"},"rating":7}'
                        )
                    ],
                    provider_id="openai",
                    model_id="openai/gpt-4o:responses",
                    provider_model_name="gpt-4o:responses",
                    raw_message=[
                        {
                            "id": "msg_089d7482d1faaec10068f966e1afec8196ada045413e3b10a2",
                            "content": [
                                {
                                    "annotations": [],
                                    "text": '{"title":"THE NAME OF THE WIND","author":{"first_name":"Patrick","last_name":"Rothfuss"},"rating":7}',
                                    "type": "output_text",
                                    "logprobs": [],
                                }
                            ],
                            "role": "assistant",
                            "status": "completed",
                            "type": "message",
                        }
                    ],
                ),
            ],
            "format": {
                "name": "Book",
                "description": "A book with a rating. The title should be in all caps!",
                "schema": {
                    "$defs": {
                        "Author": {
                            "description": "The author of a book.",
                            "properties": {
                                "first_name": {"title": "First Name", "type": "string"},
                                "last_name": {"title": "Last Name", "type": "string"},
                            },
                            "required": ["first_name", "last_name"],
                            "title": "Author",
                            "type": "object",
                            "additionalProperties": False,
                        }
                    },
                    "description": "A book with a rating. The title should be in all caps!",
                    "properties": {
                        "title": {"title": "Title", "type": "string"},
                        "author": {"$ref": "#/$defs/Author"},
                        "rating": {
                            "description": "For testing purposes, the rating should be 7",
                            "title": "Rating",
                            "type": "integer",
                        },
                    },
                    "required": ["title", "author", "rating"],
                    "title": "Book",
                    "type": "object",
                },
                "mode": "strict",
                "formatting_instructions": None,
            },
            "tools": [],
            "usage": {
                "input_tokens": 166,
                "output_tokens": 30,
                "cache_read_tokens": 0,
                "cache_write_tokens": 0,
                "reasoning_tokens": 0,
                "provider_tool_usage": None,
                "raw": "None",
                "total_tokens": 196,
            },
            "n_chunks": 31,
        }
    }
)
async_stream_snapshot = snapshot(
    {
        "response": {
            "provider_id": "openai",
            "model_id": "openai/gpt-4o:responses",
            "provider_model_name": "gpt-4o:responses",
            "finish_reason": None,
            "messages": [
                UserMessage(
                    content=[
                        Text(
                            text="Please recommend the most popular book by Patrick Rothfuss"
                        )
                    ]
                ),
                AssistantMessage(
                    content=[
                        Text(
                            text='{"title":"THE NAME OF THE WIND","author":{"first_name":"Patrick","last_name":"Rothfuss"},"rating":7}'
                        )
                    ],
                    provider_id="openai",
                    model_id="openai/gpt-4o:responses",
                    provider_model_name="gpt-4o:responses",
                    raw_message=[
                        {
                            "id": "msg_09e0316a14ecf6580068f966e900988193835e59c7f4b80e25",
                            "content": [
                                {
                                    "annotations": [],
                                    "text": '{"title":"THE NAME OF THE WIND","author":{"first_name":"Patrick","last_name":"Rothfuss"},"rating":7}',
                                    "type": "output_text",
                                    "logprobs": [],
                                }
                            ],
                            "role": "assistant",
                            "status": "completed",
                            "type": "message",
                        }
                    ],
                ),
            ],
            "format": {
                "name": "Book",
                "description": "A book with a rating. The title should be in all caps!",
                "schema": {
                    "$defs": {
                        "Author": {
                            "description": "The author of a book.",
                            "properties": {
                                "first_name": {"title": "First Name", "type": "string"},
                                "last_name": {"title": "Last Name", "type": "string"},
                            },
                            "required": ["first_name", "last_name"],
                            "title": "Author",
                            "type": "object",
                            "additionalProperties": False,
                        }
                    },
                    "description": "A book with a rating. The title should be in all caps!",
                    "properties": {
                        "title": {"title": "Title", "type": "string"},
                        "author": {"$ref": "#/$defs/Author"},
                        "rating": {
                            "description": "For testing purposes, the rating should be 7",
                            "title": "Rating",
                            "type": "integer",
                        },
                    },
                    "required": ["title", "author", "rating"],
                    "title": "Book",
                    "type": "object",
                },
                "mode": "strict",
                "formatting_instructions": None,
            },
            "tools": [],
            "usage": {
                "input_tokens": 166,
                "output_tokens": 30,
                "cache_read_tokens": 0,
                "cache_write_tokens": 0,
                "reasoning_tokens": 0,
                "provider_tool_usage": None,
                "raw": "None",
                "total_tokens": 196,
            },
            "n_chunks": 31,
        }
    }
)



from mirascope import llm


@llm.call(
    "anthropic/claude-sonnet-4-5",
    thinking={"level": "high", "include_thoughts": True},
)
def solve(problem: str) -> str:
    return problem


response = solve("Prove that the square root of 2 is irrational.")
print(response.pretty())



"""Google provider implementation."""

from __future__ import annotations

from collections.abc import Sequence
from typing import TYPE_CHECKING
from typing_extensions import Unpack

from google.genai import Client
from google.genai.types import HttpOptions

from ...context import Context, DepsT
from ...formatting import FormatSpec, FormattableT
from ...messages import Message
from ...responses import (
    AsyncContextResponse,
    AsyncContextStreamResponse,
    AsyncResponse,
    AsyncStreamResponse,
    ContextResponse,
    ContextStreamResponse,
    Response,
    StreamResponse,
)
from ...tools import (
    AsyncContextToolkit,
    AsyncToolkit,
    ContextToolkit,
    Toolkit,
)
from ..base import BaseProvider
from . import _utils
from .model_id import GoogleModelId, model_name

if TYPE_CHECKING:
    from ...models import Params


class GoogleProvider(BaseProvider[Client]):
    """The client for the Google LLM model."""

    id = "google"
    default_scope = "google/"
    error_map = _utils.GOOGLE_ERROR_MAP

    def __init__(
        self, *, api_key: str | None = None, base_url: str | None = None
    ) -> None:
        """Initialize the Google client."""
        http_options = None
        if base_url:
            http_options = HttpOptions(
                base_url=base_url,
                headers={"Authorization": f"Bearer {api_key}"},
            )

        self.client = Client(api_key=api_key, http_options=http_options)

    def get_error_status(self, e: Exception) -> int | None:
        """Extract HTTP status code from Google exception."""
        return getattr(e, "code", None)

    def _call(
        self,
        *,
        model_id: GoogleModelId,
        messages: Sequence[Message],
        toolkit: Toolkit,
        format: FormatSpec[FormattableT] | None = None,
        **params: Unpack[Params],
    ) -> Response | Response[FormattableT]:
        """Generate an `llm.Response` by synchronously calling the Google GenAI API.

        Args:
            model_id: Model identifier to use.
            messages: Messages to send to the LLM.
            tools: Optional tools that the model may invoke.
            format: Optional response format specifier.
            **params: Additional parameters to configure output (e.g. temperature). See `llm.Params`.

        Returns:
            An `llm.Response` object containing the LLM-generated content.
        """
        input_messages, format, kwargs = _utils.encode_request(
            model_id=model_id,
            messages=messages,
            tools=toolkit,
            format=format,
            params=params,
        )
        google_response = self.client.models.generate_content(**kwargs)

        include_thoughts = _utils.get_include_thoughts(params)
        assistant_message, finish_reason, usage = _utils.decode_response(
            google_response, model_id, include_thoughts=include_thoughts
        )

        return Response(
            raw=google_response,
            provider_id="google",
            model_id=model_id,
            provider_model_name=model_name(model_id),
            params=params,
            tools=toolkit,
            input_messages=input_messages,
            assistant_message=assistant_message,
            finish_reason=finish_reason,
            usage=usage,
            format=format,
        )

    def _context_call(
        self,
        *,
        ctx: Context[DepsT],
        model_id: GoogleModelId,
        messages: Sequence[Message],
        toolkit: ContextToolkit[DepsT],
        format: FormatSpec[FormattableT] | None = None,
        **params: Unpack[Params],
    ) -> ContextResponse[DepsT, None] | ContextResponse[DepsT, FormattableT]:
        """Generate an `llm.ContextResponse` by synchronously calling the Google GenAI API.

        Args:
            ctx: Context object with dependencies for tools.
            model_id: Model identifier to use.
            messages: Messages to send to the LLM.
            tools: Optional tools that the model may invoke.
            format: Optional response format specifier.
            **params: Additional parameters to configure output (e.g. temperature). See `llm.Params`.

        Returns:
            An `llm.ContextResponse` object containing the LLM-generated content.
        """
        input_messages, format, kwargs = _utils.encode_request(
            model_id=model_id,
            messages=messages,
            tools=toolkit,
            format=format,
            params=params,
        )
        google_response = self.client.models.generate_content(**kwargs)

        include_thoughts = _utils.get_include_thoughts(params)
        assistant_message, finish_reason, usage = _utils.decode_response(
            google_response, model_id, include_thoughts=include_thoughts
        )

        return ContextResponse(
            raw=google_response,
            provider_id="google",
            model_id=model_id,
            provider_model_name=model_name(model_id),
            params=params,
            tools=toolkit,
            input_messages=input_messages,
            assistant_message=assistant_message,
            finish_reason=finish_reason,
            usage=usage,
            format=format,
        )

    async def _call_async(
        self,
        *,
        model_id: GoogleModelId,
        messages: Sequence[Message],
        toolkit: AsyncToolkit,
        format: FormatSpec[FormattableT] | None = None,
        **params: Unpack[Params],
    ) -> AsyncResponse | AsyncResponse[FormattableT]:
        """Generate an `llm.AsyncResponse` by asynchronously calling the Google GenAI API.

        Args:
            model_id: Model identifier to use.
            messages: Messages to send to the LLM.
            tools: Optional tools that the model may invoke.
            format: Optional response format specifier.
            **params: Additional parameters to configure output (e.g. temperature). See `llm.Params`.

        Returns:
            An `llm.AsyncResponse` object containing the LLM-generated content.
        """
        input_messages, format, kwargs = _utils.encode_request(
            model_id=model_id,
            messages=messages,
            tools=toolkit,
            format=format,
            params=params,
        )
        google_response = await self.client.aio.models.generate_content(**kwargs)

        include_thoughts = _utils.get_include_thoughts(params)
        assistant_message, finish_reason, usage = _utils.decode_response(
            google_response, model_id, include_thoughts=include_thoughts
        )

        return AsyncResponse(
            raw=google_response,
            provider_id="google",
            model_id=model_id,
            provider_model_name=model_name(model_id),
            params=params,
            tools=toolkit,
            input_messages=input_messages,
            assistant_message=assistant_message,
            finish_reason=finish_reason,
            usage=usage,
            format=format,
        )

    async def _context_call_async(
        self,
        *,
        ctx: Context[DepsT],
        model_id: GoogleModelId,
        messages: Sequence[Message],
        toolkit: AsyncContextToolkit[DepsT],
        format: FormatSpec[FormattableT] | None = None,
        **params: Unpack[Params],
    ) -> AsyncContextResponse[DepsT, None] | AsyncContextResponse[DepsT, FormattableT]:
        """Generate an `llm.AsyncContextResponse` by asynchronously calling the Google GenAI API.

        Args:
            ctx: Context object with dependencies for tools.
            model_id: Model identifier to use.
            messages: Messages to send to the LLM.
            tools: Optional tools that the model may invoke.
            format: Optional response format specifier.
            **params: Additional parameters to configure output (e.g. temperature). See `llm.Params`.

        Returns:
            An `llm.AsyncContextResponse` object containing the LLM-generated content.
        """
        input_messages, format, kwargs = _utils.encode_request(
            model_id=model_id,
            messages=messages,
            tools=toolkit,
            format=format,
            params=params,
        )
        google_response = await self.client.aio.models.generate_content(**kwargs)

        include_thoughts = _utils.get_include_thoughts(params)
        assistant_message, finish_reason, usage = _utils.decode_response(
            google_response, model_id, include_thoughts=include_thoughts
        )

        return AsyncContextResponse(
            raw=google_response,
            provider_id="google",
            model_id=model_id,
            provider_model_name=model_name(model_id),
            params=params,
            tools=toolkit,
            input_messages=input_messages,
            assistant_message=assistant_message,
            finish_reason=finish_reason,
            usage=usage,
            format=format,
        )

    def _stream(
        self,
        *,
        model_id: GoogleModelId,
        messages: Sequence[Message],
        toolkit: Toolkit,
        format: FormatSpec[FormattableT] | None = None,
        **params: Unpack[Params],
    ) -> StreamResponse | StreamResponse[FormattableT]:
        """Generate an `llm.StreamResponse` by synchronously streaming from the Google GenAI API.

        Args:
            model_id: Model identifier to use.
            messages: Messages to send to the LLM.
            tools: Optional tools that the model may invoke.
            format: Optional response format specifier.
            **params: Additional parameters to configure output (e.g. temperature). See `llm.Params`.

        Returns:
            An `llm.StreamResponse` object for iterating over the LLM-generated content.
        """
        input_messages, format, kwargs = _utils.encode_request(
            model_id=model_id,
            messages=messages,
            tools=toolkit,
            format=format,
            params=params,
        )

        google_stream = self.client.models.generate_content_stream(**kwargs)

        include_thoughts = _utils.get_include_thoughts(params)
        chunk_iterator = _utils.decode_stream(
            google_stream, include_thoughts=include_thoughts
        )

        return StreamResponse(
            provider_id="google",
            model_id=model_id,
            provider_model_name=model_name(model_id),
            params=params,
            tools=toolkit,
            input_messages=input_messages,
            chunk_iterator=chunk_iterator,
            format=format,
        )

    def _context_stream(
        self,
        *,
        ctx: Context[DepsT],
        model_id: GoogleModelId,
        messages: Sequence[Message],
        toolkit: ContextToolkit[DepsT],
        format: FormatSpec[FormattableT] | None = None,
        **params: Unpack[Params],
    ) -> ContextStreamResponse[DepsT] | ContextStreamResponse[DepsT, FormattableT]:
        """Generate an `llm.ContextStreamResponse` by synchronously streaming from the Google GenAI API.

        Args:
            ctx: Context object with dependencies for tools.
            model_id: Model identifier to use.
            messages: Messages to send to the LLM.
            tools: Optional tools that the model may invoke.
            format: Optional response format specifier.
            **params: Additional parameters to configure output (e.g. temperature). See `llm.Params`.

        Returns:
            An `llm.ContextStreamResponse` object for iterating over the LLM-generated content.
        """
        input_messages, format, kwargs = _utils.encode_request(
            model_id=model_id,
            messages=messages,
            tools=toolkit,
            format=format,
            params=params,
        )

        google_stream = self.client.models.generate_content_stream(**kwargs)

        include_thoughts = _utils.get_include_thoughts(params)
        chunk_iterator = _utils.decode_stream(
            google_stream, include_thoughts=include_thoughts
        )

        return ContextStreamResponse(
            provider_id="google",
            model_id=model_id,
            provider_model_name=model_name(model_id),
            params=params,
            tools=toolkit,
            input_messages=input_messages,
            chunk_iterator=chunk_iterator,
            format=format,
        )

    async def _stream_async(
        self,
        *,
        model_id: GoogleModelId,
        messages: Sequence[Message],
        toolkit: AsyncToolkit,
        format: FormatSpec[FormattableT] | None = None,
        **params: Unpack[Params],
    ) -> AsyncStreamResponse | AsyncStreamResponse[FormattableT]:
        """Generate an `llm.AsyncStreamResponse` by asynchronously streaming from the Google GenAI API.

        Args:
            model_id: Model identifier to use.
            messages: Messages to send to the LLM.
            tools: Optional tools that the model may invoke.
            format: Optional response format specifier.
            **params: Additional parameters to configure output (e.g. temperature). See `llm.Params`.

        Returns:
            An `llm.AsyncStreamResponse` object for asynchronously iterating over the LLM-generated content.
        """
        input_messages, format, kwargs = _utils.encode_request(
            model_id=model_id,
            messages=messages,
            tools=toolkit,
            format=format,
            params=params,
        )

        google_stream = await self.client.aio.models.generate_content_stream(**kwargs)

        include_thoughts = _utils.get_include_thoughts(params)
        chunk_iterator = _utils.decode_async_stream(
            google_stream, include_thoughts=include_thoughts
        )

        return AsyncStreamResponse(
            provider_id="google",
            model_id=model_id,
            provider_model_name=model_name(model_id),
            params=params,
            tools=toolkit,
            input_messages=input_messages,
            chunk_iterator=chunk_iterator,
            format=format,
        )

    async def _context_stream_async(
        self,
        *,
        ctx: Context[DepsT],
        model_id: GoogleModelId,
        messages: Sequence[Message],
        toolkit: AsyncContextToolkit[DepsT],
        format: FormatSpec[FormattableT] | None = None,
        **params: Unpack[Params],
    ) -> (
        AsyncContextStreamResponse[DepsT]
        | AsyncContextStreamResponse[DepsT, FormattableT]
    ):
        """Generate an `llm.AsyncContextStreamResponse` by asynchronously streaming from the Google GenAI API.

        Args:
            ctx: Context object with dependencies for tools.
            model_id: Model identifier to use.
            messages: Messages to send to the LLM.
            tools: Optional tools that the model may invoke.
            format: Optional response format specifier.
            **params: Additional parameters to configure output (e.g. temperature). See `llm.Params`.

        Returns:
            An `llm.AsyncContextStreamResponse` object for asynchronously iterating over the LLM-generated content.
        """
        input_messages, format, kwargs = _utils.encode_request(
            model_id=model_id,
            messages=messages,
            tools=toolkit,
            format=format,
            params=params,
        )

        google_stream = await self.client.aio.models.generate_content_stream(**kwargs)

        include_thoughts = _utils.get_include_thoughts(params)
        chunk_iterator = _utils.decode_async_stream(
            google_stream, include_thoughts=include_thoughts
        )

        return AsyncContextStreamResponse(
            provider_id="google",
            model_id=model_id,
            provider_model_name=model_name(model_id),
            params=params,
            tools=toolkit,
            input_messages=input_messages,
            chunk_iterator=chunk_iterator,
            format=format,
        )



"""Shared fixtures for retry tests."""

from collections.abc import Generator

import pytest

from mirascope import llm

from .mock_provider import MockProvider

SERVER_ERROR = llm.ServerError("server error", provider="test")
CONNECTION_ERROR = llm.ConnectionError("connection failed", provider="test")
RATE_LIMIT_ERROR = llm.RateLimitError("rate limited", provider="test")
TIMEOUT_ERROR = llm.TimeoutError("timeout", provider="test")

# Common exception sequences used across tests
DEFAULT_RETRYABLE_EXCEPTIONS: list[llm.Error] = [
    SERVER_ERROR,
    CONNECTION_ERROR,
    RATE_LIMIT_ERROR,
    TIMEOUT_ERROR,
]

RESUME_TEST_EXCEPTIONS: list[llm.Error] = [
    CONNECTION_ERROR,
    RATE_LIMIT_ERROR,
]


@pytest.fixture
def mock_provider() -> Generator[MockProvider, None, None]:
    """Fixture that provides a MockProvider and cleans up after the test."""
    llm.reset_provider_registry()
    provider = MockProvider()
    llm.register_provider(provider, scope="mock/")
    yield provider
    llm.reset_provider_registry()


@pytest.fixture(autouse=True)
def mock_sleep(monkeypatch: pytest.MonkeyPatch) -> list[float]:
    """Auto-mock time.sleep and asyncio.sleep to avoid actual delays in tests.

    Returns a list that captures all sleep durations for assertions.
    """
    sleep_calls: list[float] = []

    def sync_sleep(seconds: float) -> None:
        sleep_calls.append(seconds)

    async def async_sleep(seconds: float) -> None:
        sleep_calls.append(seconds)

    monkeypatch.setattr("mirascope.llm.retries.retry_models.time.sleep", sync_sleep)
    monkeypatch.setattr("mirascope.llm.retries.retry_models.asyncio.sleep", async_sleep)
    return sleep_calls
